'use strict';

const isSymbol = require('./isSymbol-9b1c8e84.js');
const utils = require('./utils-601945ce.js');

const COMPLEX_RUBRIC = 'complex-rubric';
/**
 * Allows you to modify the markup for a package that is present in elements/model but
 * missing from markup.
 * @param content the pie content
 * @param npmPackage the npm package
 * @param template a callback function for modifing the markup
 */
const addMarkupForPackage = (content, npmPackage, template) => {
    const out = isSymbol.cloneDeep(content);
    const elem = out && utils.elementForPackage(out, npmPackage);
    if (elem && utils.modelsForPackage(out, npmPackage).length > 0) {
        const match = out.markup.match(new RegExp(elem));
        if (out.markup !== null && !match) {
            const id = out.models && out.models.find(m => m.element === elem).id;
            if (id) {
                out.markup = template(id, elem, out.markup);
            }
        }
    }
    return out;
};
/**
 * This function takes the config and configSettings and determines if there should/shouldn't be a complex-rubric in the config
 * depending on configSettings (withRubric.forceEnabled) and config (model.rubricEnabled)
 * @param content
 * @param configSettings
 */
const complexRubricChecks = (content, configSettings = {}) => {
    const elements = content.elements || {};
    const elementsKeys = Object.keys(elements || {});
    // tODO make sure not to use versions (eg: @1.1.0, @latest)
    const elementsValues = Object.values(elements || {}).map(elementValue => utils.getPackageWithoutVersion(elementValue));
    // complexRubricElements: those elements in config where item type is @pie-element/complex-rubric
    const complexRubricElements = elementsKeys.filter(key => elements[key] && elements[key].indexOf(COMPLEX_RUBRIC) >= 0);
    // complexRubricItemsLength: how many complex-rubric elements are declared
    const complexRubricItemsLength = complexRubricElements.length;
    const complexRubricModels = (content.models || []).filter(model => model.element.indexOf(COMPLEX_RUBRIC) >= 0);
    const complexRubricModelsLength = complexRubricModels.length;
    if (complexRubricItemsLength === elementsKeys.length) {
        // if item config ONLY has complex-rubrics, then all the steps below are not necessary
        // this is added to treat the special case of testing complex-rubric (as single item type) in pie-website
        return {};
    }
    // if at least one model has rubricEnabled = true, then we should have complex-rubric in the config
    let shouldHaveComplexRubric = !!(content.models || []).filter(model => model.rubricEnabled).length;
    // if there's at least one configuration where withRubric has forceEnabled: true
    const shouldForceEnableComplexRubric = configuration => configuration && configuration.withRubric && configuration.withRubric.forceEnabled;
    let shouldHaveForcedComplexRubric = !!(elementsValues.filter(elementsKey => shouldForceEnableComplexRubric(configSettings[elementsKey])).length);
    shouldHaveComplexRubric = shouldHaveComplexRubric || shouldHaveForcedComplexRubric;
    return {
        shouldAddComplexRubric: shouldHaveComplexRubric && !complexRubricModelsLength,
        shouldRemoveComplexRubric: !shouldHaveComplexRubric && complexRubricModelsLength,
        complexRubricElements
    };
};
/**
 * Removes complex-rubric and rubric html from markup.
 */
const removeComplexRubricFromMarkup = (content, complexRubricElements, doc) => {
    const tempDiv = doc.createElement("div");
    tempDiv.innerHTML = content.markup;
    const elsWithId = tempDiv.querySelectorAll("[id]");
    const deletedComplexRubricItemIds = [];
    (elsWithId || []).forEach(el => {
        const pieElName = el.tagName.toLowerCase().split("-config")[0];
        // we have to remove the complex-rubric item from the markup
        if (complexRubricElements.includes(pieElName)) {
            try {
                deletedComplexRubricItemIds.push(el.id);
                tempDiv.querySelector(`#${el.id}`).remove();
            }
            catch (e) {
                console.log(e.toString());
            }
        }
    });
    const newMarkup = tempDiv.innerHTML;
    tempDiv.remove();
    return {
        markupWithoutComplexRubric: newMarkup,
        deletedComplexRubricItemIds
    };
};
/**
 * Adds complex-rubric html to markup.
 * @param content
 */
const addComplexRubric = (content) => {
    return addMarkupForPackage(isSymbol.cloneDeep(content), `@pie-element/${COMPLEX_RUBRIC}`, (id, tag, markup) => {
        return `${markup}<${tag} id="${id}"></${tag}>`;
    });
};
/**
 * Adds rubric html to markup.
 * @param content
 */
const addRubric = (content) => {
    return addMarkupForPackage(isSymbol.cloneDeep(content), "@pie-element/rubric", (id, tag, markup) => {
        return `
    ${markup}
    <div style="width: 75%">
      <${tag} id="${id}"></${tag}>
    </div>
    `;
    });
};
/**
 * Adds multi-trait-rubric html to markup.
 * @param content
 */
const addMultiTraitRubric = (content) => {
    return addMarkupForPackage(isSymbol.cloneDeep(content), "@pie-element/multi-trait-rubric", (id, tag, markup) => {
        return `
    ${markup}
    <div style="margin-top: 20px;">
      <${tag} id="${id}"></${tag}>
    </div>
    `;
    });
};
/**
 * Adds the provided package to the provided PieContent Object's `elements` and `models` properties.
 *
 * @param content the PieContent for rendering
 * @param packageToAdd the NPM Package to add to the content config
 * @param model optional the PieModel to add, `id` and `element` properties will be replaced by this function if present
 */
const addPackageToContent = (content, packageToAdd, model) => {
    if (packageToAdd && !utils.elementsHasPackage(content.elements, packageToAdd)) {
        model = model ? model : {};
        // add package
        model.id = utils.pieShortIdGenerator();
        const elementName = utils.pieShortIdGenerator();
        model.element = elementName;
        content.models && content.models.push(model);
        content.elements && (content.elements[elementName] = packageToAdd);
        return content;
    }
    return null;
};
/**
 * Adds the provided model to the provided PieContent Object's `models` properties.
 *
 * @param content the PieContent for rendering
 * @param element the name of the package converted to element name
 * @param model optional the PieModel to add, `id` and `element` properties will be replaced by this function if present
 */
const addModelToContent = (content, element, model) => {
    model = model ? model : {};
    model.id = utils.pieShortIdGenerator();
    model.element = element;
    content.models && content.models.push(model);
    return content;
};

/** Used to match property names within property paths. */
var reIsDeepProp = /\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,
    reIsPlainProp = /^\w*$/;

/**
 * Checks if `value` is a property name and not a property path.
 *
 * @private
 * @param {*} value The value to check.
 * @param {Object} [object] The object to query keys on.
 * @returns {boolean} Returns `true` if `value` is a property name, else `false`.
 */
function isKey(value, object) {
  if (isSymbol.isArray(value)) {
    return false;
  }
  var type = typeof value;
  if (type == 'number' || type == 'symbol' || type == 'boolean' ||
      value == null || isSymbol.isSymbol(value)) {
    return true;
  }
  return reIsPlainProp.test(value) || !reIsDeepProp.test(value) ||
    (object != null && value in Object(object));
}

var _isKey = isKey;

/** Error message constants. */
var FUNC_ERROR_TEXT = 'Expected a function';

/**
 * Creates a function that memoizes the result of `func`. If `resolver` is
 * provided, it determines the cache key for storing the result based on the
 * arguments provided to the memoized function. By default, the first argument
 * provided to the memoized function is used as the map cache key. The `func`
 * is invoked with the `this` binding of the memoized function.
 *
 * **Note:** The cache is exposed as the `cache` property on the memoized
 * function. Its creation may be customized by replacing the `_.memoize.Cache`
 * constructor with one whose instances implement the
 * [`Map`](http://ecma-international.org/ecma-262/7.0/#sec-properties-of-the-map-prototype-object)
 * method interface of `clear`, `delete`, `get`, `has`, and `set`.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Function
 * @param {Function} func The function to have its output memoized.
 * @param {Function} [resolver] The function to resolve the cache key.
 * @returns {Function} Returns the new memoized function.
 * @example
 *
 * var object = { 'a': 1, 'b': 2 };
 * var other = { 'c': 3, 'd': 4 };
 *
 * var values = _.memoize(_.values);
 * values(object);
 * // => [1, 2]
 *
 * values(other);
 * // => [3, 4]
 *
 * object.a = 2;
 * values(object);
 * // => [1, 2]
 *
 * // Modify the result cache.
 * values.cache.set(object, ['a', 'b']);
 * values(object);
 * // => ['a', 'b']
 *
 * // Replace `_.memoize.Cache`.
 * _.memoize.Cache = WeakMap;
 */
function memoize(func, resolver) {
  if (typeof func != 'function' || (resolver != null && typeof resolver != 'function')) {
    throw new TypeError(FUNC_ERROR_TEXT);
  }
  var memoized = function() {
    var args = arguments,
        key = resolver ? resolver.apply(this, args) : args[0],
        cache = memoized.cache;

    if (cache.has(key)) {
      return cache.get(key);
    }
    var result = func.apply(this, args);
    memoized.cache = cache.set(key, result) || cache;
    return result;
  };
  memoized.cache = new (memoize.Cache || isSymbol.MapCache);
  return memoized;
}

// Expose `MapCache`.
memoize.Cache = isSymbol.MapCache;

var memoize_1 = memoize;

/** Used as the maximum memoize cache size. */
var MAX_MEMOIZE_SIZE = 500;

/**
 * A specialized version of `_.memoize` which clears the memoized function's
 * cache when it exceeds `MAX_MEMOIZE_SIZE`.
 *
 * @private
 * @param {Function} func The function to have its output memoized.
 * @returns {Function} Returns the new memoized function.
 */
function memoizeCapped(func) {
  var result = memoize_1(func, function(key) {
    if (cache.size === MAX_MEMOIZE_SIZE) {
      cache.clear();
    }
    return key;
  });

  var cache = result.cache;
  return result;
}

var _memoizeCapped = memoizeCapped;

/** Used to match property names within property paths. */
var rePropName = /[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g;

/** Used to match backslashes in property paths. */
var reEscapeChar = /\\(\\)?/g;

/**
 * Converts `string` to a property path array.
 *
 * @private
 * @param {string} string The string to convert.
 * @returns {Array} Returns the property path array.
 */
var stringToPath = _memoizeCapped(function(string) {
  var result = [];
  if (string.charCodeAt(0) === 46 /* . */) {
    result.push('');
  }
  string.replace(rePropName, function(match, number, quote, subString) {
    result.push(quote ? subString.replace(reEscapeChar, '$1') : (number || match));
  });
  return result;
});

var _stringToPath = stringToPath;

/**
 * A specialized version of `_.map` for arrays without support for iteratee
 * shorthands.
 *
 * @private
 * @param {Array} [array] The array to iterate over.
 * @param {Function} iteratee The function invoked per iteration.
 * @returns {Array} Returns the new mapped array.
 */
function arrayMap(array, iteratee) {
  var index = -1,
      length = array == null ? 0 : array.length,
      result = Array(length);

  while (++index < length) {
    result[index] = iteratee(array[index], index, array);
  }
  return result;
}

var _arrayMap = arrayMap;

/** Used as references for various `Number` constants. */
var INFINITY = 1 / 0;

/** Used to convert symbols to primitives and strings. */
var symbolProto = isSymbol.Symbol ? isSymbol.Symbol.prototype : undefined,
    symbolToString = symbolProto ? symbolProto.toString : undefined;

/**
 * The base implementation of `_.toString` which doesn't convert nullish
 * values to empty strings.
 *
 * @private
 * @param {*} value The value to process.
 * @returns {string} Returns the string.
 */
function baseToString(value) {
  // Exit early for strings to avoid a performance hit in some environments.
  if (typeof value == 'string') {
    return value;
  }
  if (isSymbol.isArray(value)) {
    // Recursively convert values (susceptible to call stack limits).
    return _arrayMap(value, baseToString) + '';
  }
  if (isSymbol.isSymbol(value)) {
    return symbolToString ? symbolToString.call(value) : '';
  }
  var result = (value + '');
  return (result == '0' && (1 / value) == -INFINITY) ? '-0' : result;
}

var _baseToString = baseToString;

/**
 * Converts `value` to a string. An empty string is returned for `null`
 * and `undefined` values. The sign of `-0` is preserved.
 *
 * @static
 * @memberOf _
 * @since 4.0.0
 * @category Lang
 * @param {*} value The value to convert.
 * @returns {string} Returns the converted string.
 * @example
 *
 * _.toString(null);
 * // => ''
 *
 * _.toString(-0);
 * // => '-0'
 *
 * _.toString([1, 2, 3]);
 * // => '1,2,3'
 */
function toString(value) {
  return value == null ? '' : _baseToString(value);
}

var toString_1 = toString;

/**
 * Casts `value` to a path array if it's not one.
 *
 * @private
 * @param {*} value The value to inspect.
 * @param {Object} [object] The object to query keys on.
 * @returns {Array} Returns the cast property path array.
 */
function castPath(value, object) {
  if (isSymbol.isArray(value)) {
    return value;
  }
  return _isKey(value, object) ? [value] : _stringToPath(toString_1(value));
}

var _castPath = castPath;

/** Used as references for various `Number` constants. */
var INFINITY$1 = 1 / 0;

/**
 * Converts `value` to a string key if it's not a string or symbol.
 *
 * @private
 * @param {*} value The value to inspect.
 * @returns {string|symbol} Returns the key.
 */
function toKey(value) {
  if (typeof value == 'string' || isSymbol.isSymbol(value)) {
    return value;
  }
  var result = (value + '');
  return (result == '0' && (1 / value) == -INFINITY$1) ? '-0' : result;
}

var _toKey = toKey;

/**
 * The base implementation of `_.get` without support for default values.
 *
 * @private
 * @param {Object} object The object to query.
 * @param {Array|string} path The path of the property to get.
 * @returns {*} Returns the resolved value.
 */
function baseGet(object, path) {
  path = _castPath(path, object);

  var index = 0,
      length = path.length;

  while (object != null && index < length) {
    object = object[_toKey(path[index++])];
  }
  return (index && index == length) ? object : undefined;
}

var _baseGet = baseGet;

/**
 * Gets the value at `path` of `object`. If the resolved value is
 * `undefined`, the `defaultValue` is returned in its place.
 *
 * @static
 * @memberOf _
 * @since 3.7.0
 * @category Object
 * @param {Object} object The object to query.
 * @param {Array|string} path The path of the property to get.
 * @param {*} [defaultValue] The value returned for `undefined` resolved values.
 * @returns {*} Returns the resolved value.
 * @example
 *
 * var object = { 'a': [{ 'b': { 'c': 3 } }] };
 *
 * _.get(object, 'a[0].b.c');
 * // => 3
 *
 * _.get(object, ['a', '0', 'b', 'c']);
 * // => 3
 *
 * _.get(object, 'a.b.c', 'default');
 * // => 'default'
 */
function get(object, path, defaultValue) {
  var result = object == null ? undefined : _baseGet(object, path);
  return result === undefined ? defaultValue : result;
}

var get_1 = get;

exports.COMPLEX_RUBRIC = COMPLEX_RUBRIC;
exports.addComplexRubric = addComplexRubric;
exports.addModelToContent = addModelToContent;
exports.addMultiTraitRubric = addMultiTraitRubric;
exports.addPackageToContent = addPackageToContent;
exports.addRubric = addRubric;
exports.arrayMap = _arrayMap;
exports.baseGet = _baseGet;
exports.castPath = _castPath;
exports.complexRubricChecks = complexRubricChecks;
exports.get = get_1;
exports.get_1 = get_1;
exports.isKey = _isKey;
exports.removeComplexRubricFromMarkup = removeComplexRubricFromMarkup;
exports.toKey = _toKey;
