'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const core = require('./core-8e67a5c9.js');
const version$1 = require('./version-52296af9.js');
require('./_commonjsHelpers-681a8719.js');
require('./index-a93bde69.js');
const isSymbol = require('./isSymbol-9b1c8e84.js');
const toNumber = require('./toNumber-f3d43ddd.js');

const isMethodAndPath = (m) => m && m.path && m.method;
const toMethodAndPath = (m) => {
    if (isMethodAndPath(m)) {
        return m;
    }
    if (typeof m === "string") {
        return { method: "POST", path: m };
    }
    throw new Error(`Unknown type ${m}`);
};
class PlayerApiClient extends version$1.NewRelicEnabledClient {
    constructor(token, endpoints, optimize, retryConfig) {
        super();
        this.token = token;
        this.endpoints = endpoints;
        this.optimize = optimize;
        // Create a FetchUtils instance with tracking info
        this.fetchUtils = new version$1.FetchUtils({
            host: this.endpoints.host,
            optimize: this.optimize,
            hasToken: !!this.token
        }, retryConfig);
    }
    /**
     * Implementation of the abstract method from NewRelicEnabledClient
     */
    getTrackingBaseAttributes() {
        return {
            host: this.endpoints.host,
            optimize: this.optimize,
            hasToken: !!this.token,
            networkInfo: this.getNetworkInfo()
        };
    }
    /**
     * Make an API call with retry mechanism
     *
     * @param endpoint - The API endpoint to call (method and path)
     * @param data - The data to send in the request body (will be JSON stringified)
     * @returns A promise that resolves to the parsed response data
     * @throws Error if all retry attempts fail
     */
    async call(endpoint, data) {
        return this.fetchUtils.callApi(`${this.endpoints.host}${endpoint.path}`, endpoint.method, this.token, data);
    }
    load(itemId, sessionId, env, assignmentId, overrides) {
        const params = {
            itemId,
            sessionId,
            assignmentId,
            env
        };
        if (overrides) {
            params.overrides = overrides;
        }
        return this.call(toMethodAndPath(this.endpoints.load), params);
    }
    saveSession(session, env) {
        return this.call(toMethodAndPath(this.endpoints.saveSession), {
            data: session.data,
            sessionId: session.id,
            env
        });
    }
    model(session, env, overrides) {
        const params = {
            sessionId: session.id,
            data: session.data,
            env
        };
        if (overrides) {
            params.overrides = overrides;
        }
        return this.call(toMethodAndPath(this.endpoints.model), params);
    }
    score(session, env, overrides) {
        const params = {
            sessionId: session.id,
            data: session.data,
            env,
            overrides
        };
        if (overrides) {
            params.overrides = overrides;
        }
        return this.call(toMethodAndPath(this.endpoints.score), params);
    }
}

/**
 * Gets the timestamp of the number of milliseconds that have elapsed since
 * the Unix epoch (1 January 1970 00:00:00 UTC).
 *
 * @static
 * @memberOf _
 * @since 2.4.0
 * @category Date
 * @returns {number} Returns the timestamp.
 * @example
 *
 * _.defer(function(stamp) {
 *   console.log(_.now() - stamp);
 * }, _.now());
 * // => Logs the number of milliseconds it took for the deferred invocation.
 */
var now = function() {
  return isSymbol.root.Date.now();
};

var now_1 = now;

/** Error message constants. */
var FUNC_ERROR_TEXT = 'Expected a function';

/* Built-in method references for those with the same name as other `lodash` methods. */
var nativeMax = Math.max,
    nativeMin = Math.min;

/**
 * Creates a debounced function that delays invoking `func` until after `wait`
 * milliseconds have elapsed since the last time the debounced function was
 * invoked. The debounced function comes with a `cancel` method to cancel
 * delayed `func` invocations and a `flush` method to immediately invoke them.
 * Provide `options` to indicate whether `func` should be invoked on the
 * leading and/or trailing edge of the `wait` timeout. The `func` is invoked
 * with the last arguments provided to the debounced function. Subsequent
 * calls to the debounced function return the result of the last `func`
 * invocation.
 *
 * **Note:** If `leading` and `trailing` options are `true`, `func` is
 * invoked on the trailing edge of the timeout only if the debounced function
 * is invoked more than once during the `wait` timeout.
 *
 * If `wait` is `0` and `leading` is `false`, `func` invocation is deferred
 * until to the next tick, similar to `setTimeout` with a timeout of `0`.
 *
 * See [David Corbacho's article](https://css-tricks.com/debouncing-throttling-explained-examples/)
 * for details over the differences between `_.debounce` and `_.throttle`.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Function
 * @param {Function} func The function to debounce.
 * @param {number} [wait=0] The number of milliseconds to delay.
 * @param {Object} [options={}] The options object.
 * @param {boolean} [options.leading=false]
 *  Specify invoking on the leading edge of the timeout.
 * @param {number} [options.maxWait]
 *  The maximum time `func` is allowed to be delayed before it's invoked.
 * @param {boolean} [options.trailing=true]
 *  Specify invoking on the trailing edge of the timeout.
 * @returns {Function} Returns the new debounced function.
 * @example
 *
 * // Avoid costly calculations while the window size is in flux.
 * jQuery(window).on('resize', _.debounce(calculateLayout, 150));
 *
 * // Invoke `sendMail` when clicked, debouncing subsequent calls.
 * jQuery(element).on('click', _.debounce(sendMail, 300, {
 *   'leading': true,
 *   'trailing': false
 * }));
 *
 * // Ensure `batchLog` is invoked once after 1 second of debounced calls.
 * var debounced = _.debounce(batchLog, 250, { 'maxWait': 1000 });
 * var source = new EventSource('/stream');
 * jQuery(source).on('message', debounced);
 *
 * // Cancel the trailing debounced invocation.
 * jQuery(window).on('popstate', debounced.cancel);
 */
function debounce(func, wait, options) {
  var lastArgs,
      lastThis,
      maxWait,
      result,
      timerId,
      lastCallTime,
      lastInvokeTime = 0,
      leading = false,
      maxing = false,
      trailing = true;

  if (typeof func != 'function') {
    throw new TypeError(FUNC_ERROR_TEXT);
  }
  wait = toNumber.toNumber(wait) || 0;
  if (isSymbol.isObject(options)) {
    leading = !!options.leading;
    maxing = 'maxWait' in options;
    maxWait = maxing ? nativeMax(toNumber.toNumber(options.maxWait) || 0, wait) : maxWait;
    trailing = 'trailing' in options ? !!options.trailing : trailing;
  }

  function invokeFunc(time) {
    var args = lastArgs,
        thisArg = lastThis;

    lastArgs = lastThis = undefined;
    lastInvokeTime = time;
    result = func.apply(thisArg, args);
    return result;
  }

  function leadingEdge(time) {
    // Reset any `maxWait` timer.
    lastInvokeTime = time;
    // Start the timer for the trailing edge.
    timerId = setTimeout(timerExpired, wait);
    // Invoke the leading edge.
    return leading ? invokeFunc(time) : result;
  }

  function remainingWait(time) {
    var timeSinceLastCall = time - lastCallTime,
        timeSinceLastInvoke = time - lastInvokeTime,
        timeWaiting = wait - timeSinceLastCall;

    return maxing
      ? nativeMin(timeWaiting, maxWait - timeSinceLastInvoke)
      : timeWaiting;
  }

  function shouldInvoke(time) {
    var timeSinceLastCall = time - lastCallTime,
        timeSinceLastInvoke = time - lastInvokeTime;

    // Either this is the first call, activity has stopped and we're at the
    // trailing edge, the system time has gone backwards and we're treating
    // it as the trailing edge, or we've hit the `maxWait` limit.
    return (lastCallTime === undefined || (timeSinceLastCall >= wait) ||
      (timeSinceLastCall < 0) || (maxing && timeSinceLastInvoke >= maxWait));
  }

  function timerExpired() {
    var time = now_1();
    if (shouldInvoke(time)) {
      return trailingEdge(time);
    }
    // Restart the timer.
    timerId = setTimeout(timerExpired, remainingWait(time));
  }

  function trailingEdge(time) {
    timerId = undefined;

    // Only invoke if we have `lastArgs` which means `func` has been
    // debounced at least once.
    if (trailing && lastArgs) {
      return invokeFunc(time);
    }
    lastArgs = lastThis = undefined;
    return result;
  }

  function cancel() {
    if (timerId !== undefined) {
      clearTimeout(timerId);
    }
    lastInvokeTime = 0;
    lastArgs = lastCallTime = lastThis = timerId = undefined;
  }

  function flush() {
    return timerId === undefined ? result : trailingEdge(now_1());
  }

  function debounced() {
    var time = now_1(),
        isInvoking = shouldInvoke(time);

    lastArgs = arguments;
    lastThis = this;
    lastCallTime = time;

    if (isInvoking) {
      if (timerId === undefined) {
        return leadingEdge(lastCallTime);
      }
      if (maxing) {
        // Handle invocations in a tight loop.
        clearTimeout(timerId);
        timerId = setTimeout(timerExpired, wait);
        return invokeFunc(lastCallTime);
      }
    }
    if (timerId === undefined) {
      timerId = setTimeout(timerExpired, wait);
    }
    return result;
  }
  debounced.cancel = cancel;
  debounced.flush = flush;
  return debounced;
}

var debounce_1 = debounce;

const PieApiPlayer = class {
    constructor(hostRef) {
        core.registerInstance(this, hostRef);
        this.initialized = false;
        /**
         * Use the beta controllers for pie-elements
         */
        this.optimize = false;
        /**
         * Enable screen recording with New Relic.
         * Note: not to be used in production unless for temporary use to track down a specific user problem.
         * Defaults to false
         */
        this.recordScreen = false;
        /**
         * Enable exception tracking with New Relic.
         */
        this.exceptions = false;
        /**
         * Enable tracking page actions/events with New Relic.
         * Note: Setting this to true increases data sent to New Relic, which may affect costs.
         */
        this.trackPageActions = false;
        /**
         * In evaluate mode, add a bottom border to visually separate each item in the case of a multi-item
         */
        this.showBottomBorder = false;
        /**
         * The mode to be used when player is initialized
         */
        this.initialMode = version$1.Mode.gather;
        /**
         * The role to be used when player is initialized
         */
        this.initialRole = version$1.Role.student;
        /**
         * If the item contains a passage, the player will render it by default.
         * Set this property to false to not render passage.
         */
        this.renderStimulus = true;
        /**
         * It will allow to resize passage and item width for items with stimulus
         */
        this.allowedResize = false;
        /**
         * Choose 'dev' , 'stage' or 'prod' environments for back end.
         */
        this.host = 'prod';
        /**
         * used to automatically re-fetch the bundle (in case we get a 503); defaults to false
         */
        this.reFetchBundle = false;
        /**
         * Customize markup.
         */
        this.customMarkup = '';
        /**
         * If this property is set the item will be rendered in a `preview` state by the player.
         * This means no server-side calls will be made and no reponse-session will be created/stored in pie-api.
         */
        this.preview = false;
        this.recordingStarted = core.createEvent(this, "recordingStarted", 7);
        this.saveSessionError = core.createEvent(this, "saveSessionError", 7);
        this.sessionSaved = core.createEvent(this, "sessionSaved", 7);
        this.modelReload = core.createEvent(this, "model-reload", 7);
        this.playerError = core.createEvent(this, "api-player-error", 7);
        this.sessionCreated = core.createEvent(this, "session-created", 7);
        this.doc = core.getContext(this, "document");
    }
    watchHostEnvironment(newVal) {
        if (newVal) {
            this._apiConfig = this._apiConfig || {};
            this._apiConfig.host = version$1.HOST_DEFAULTS[newVal].apiUrl;
            this._apiConfig.bundleBaseUrl = version$1.HOST_DEFAULTS[newVal].bundleBaseUrl;
        }
    }
    onCustomConfigChange(newValue) {
        console.log('customConfig now', newValue);
    }
    onCustomModelsChange(newValue) {
        console.log('customModels now', newValue);
    }
    onCustomPassageModelsChange(newValue) {
        console.log('customPassageModels now', newValue);
    }
    onCustomMarkupChange(newValue) {
        console.log('customMarkup now', newValue);
    }
    get safeConfig() {
        return Object.assign(Object.assign(Object.assign({}, version$1.PLAYER_HOST_CONFIG), this._apiConfig), this.customConfig);
    }
    /**
     * Get the retry configuration from component properties
     */
    getRetryConfig() {
        const config = {};
        if (this.maxRetryAttempts !== undefined)
            config.maxRetryAttempts = this.maxRetryAttempts;
        if (this.requestTimeoutMs !== undefined)
            config.requestTimeoutMs = this.requestTimeoutMs;
        if (this.initialRetryDelayMs !== undefined)
            config.initialRetryDelayMs = this.initialRetryDelayMs;
        if (this.maxRetryDelayMs !== undefined)
            config.maxRetryDelayMs = this.maxRetryDelayMs;
        if (this.trackPageActions !== undefined)
            config.trackPageActions = this.trackPageActions;
        return Object.keys(config).length > 0 ? config : undefined;
    }
    initPlayerClient() {
        if (this.token) {
            this.playerClient = new PlayerApiClient(this.token, this.safeConfig, this.optimize, this.getRetryConfig());
        }
    }
    async watchToken() {
        try {
            this.initPlayerClient();
            await this.load();
            /**
             * Have to call this cos playerInfo is not part of the component state?
             */
            this.el.forceUpdate();
        }
        catch (err) {
            this.trackError('Error during token update', err, {
                errorContext: 'watchToken',
                hasPlayerClient: !!this.playerClient
            });
            this.playerError.emit({
                message: 'Error during token update',
                details: err.message || err,
            });
        }
    }
    async watchItemId(newValue, oldValue) {
        if (newValue !== oldValue) {
            this.updateItemOrSession();
        }
    }
    async watchRetryConfig() {
        // Reinitialize the player client with the new retry configuration
        if (this.token) {
            this.initPlayerClient();
        }
    }
    async watchEnv(newEnv) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r;
        const updatedEnv = Object.assign(Object.assign({}, (this.player && this.player.env
            ? this.player.env
            : {})), newEnv);
        if (this.playerInfo && this.player) {
            try {
                if (!this.preview) {
                    const allModels = await this.playerClient.model(this.player.session, updatedEnv, this.getOverrides());
                    const player = this.player;
                    const models = ((_b = (_a = player) === null || _a === void 0 ? void 0 : _a.config) === null || _b === void 0 ? void 0 : _b.models) || ((_e = (_d = (_c = player) === null || _c === void 0 ? void 0 : _c.config) === null || _d === void 0 ? void 0 : _d.pie) === null || _e === void 0 ? void 0 : _e.models) || [];
                    const passageModels = ((_h = (_g = (_f = player) === null || _f === void 0 ? void 0 : _f.config) === null || _g === void 0 ? void 0 : _g.passage) === null || _h === void 0 ? void 0 : _h.models) || [];
                    // search model based on id and element
                    const findModel = (models, model) => { var _a; return (_a = models) === null || _a === void 0 ? void 0 : _a.find((m) => m.id === model.id && m.element === model.element); };
                    // filter models from allModels
                    const updatedModels = allModels.filter((model) => findModel(models, model));
                    // filter passage models from allModels
                    const updatedPassageModels = allModels.filter((model) => findModel(passageModels, model));
                    const config = this.getConfigParsed(player.config, updatedModels, updatedPassageModels);
                    player.config = config;
                }
                this.player.env = updatedEnv;
                return { error: null };
            }
            catch (e) {
                this.trackError('Environment update error', e, {
                    errorContext: 'watchEnv',
                    originalEnv: JSON.stringify(this.env || {}),
                    newEnv: JSON.stringify(newEnv || {}),
                    updatedEnv: JSON.stringify(updatedEnv || {}),
                    hasPlayerInfo: !!this.playerInfo,
                    hasPlayerConfig: !!((_j = this.player) === null || _j === void 0 ? void 0 : _j.config),
                    modelCount: ((_m = (_l = (_k = this.player) === null || _k === void 0 ? void 0 : _k.config) === null || _l === void 0 ? void 0 : _l.models) === null || _m === void 0 ? void 0 : _m.length) || ((_r = (_q = (_p = (_o = this.player) === null || _o === void 0 ? void 0 : _o.config) === null || _p === void 0 ? void 0 : _p.pie) === null || _q === void 0 ? void 0 : _q.models) === null || _r === void 0 ? void 0 : _r.length) || 0
                });
                this.playerError.emit({
                    message: 'Error updating environment',
                    details: e.message || e,
                });
                return { error: e.message };
            }
        }
    }
    watchRecordScreen(newVal, oldVal) {
        if (newVal && newVal !== oldVal) {
            if (this.newRelic && this.recordScreenIdentity) {
                // Set user identity for New Relic session
                this.newRelic.setUserId(this.recordScreenIdentity);
                // Record session start
                if (this.trackPageActions) {
                    this.newRelic.addPageAction('session_recording_started', {
                        appVersion: version$1.VERSION,
                        itemId: this.itemId,
                        sessionId: this.sessionId
                    });
                }
                this.recordingStarted.emit('Session recording started with New Relic');
            }
        }
    }
    /**
     * Return the evaluated score for the current response.
     * The api token must provide permission to authorize the use of this method.
     *
     * Note: Score does not (yet) work when in preview mode - will resolve to null if `preview` is `true`
     *
     * @param disablePartialScoring - if true all interactions will be scored as either correct or incorrect
     */
    async score(disablePartialScoring) {
        if (disablePartialScoring === true || disablePartialScoring === false) {
            console.warn('disablePartialScoring is no longer used - set env instead: {partialScoring: true|false, ...}');
        }
        if (this.preview) {
            return Promise.resolve(null);
        }
        else {
            return this.playerClient.score(this.playerInfo.session, this.env, this.getOverrides());
        }
    }
    async saveSession() {
        var _a, _b, _c, _d;
        if (this.player && this.player.session && !this.preview) {
            try {
                await this.playerClient.saveSession(this.player.session, this.env || {});
                // Track successful session save in New Relic
                if (this.newRelic && this.trackPageActions) {
                    this.newRelic.addPageAction('session_saved', {
                        sessionId: this.sessionId
                    });
                }
                this.sessionSaved.emit(this.sessionId);
            }
            catch (error) {
                console.warn('could not save session');
                this.saveSessionError.emit(error);
                this.trackError('Error saving session', error, {
                    errorContext: 'session-changed event',
                    envMode: ((_a = this.env) === null || _a === void 0 ? void 0 : _a.mode) || this.initialMode,
                    envRole: ((_b = this.env) === null || _b === void 0 ? void 0 : _b.role) || this.initialRole,
                    sessionData: JSON.stringify(((_c = this.player.session) === null || _c === void 0 ? void 0 : _c.data) || []),
                    hasSessionId: !!this.sessionId,
                    sessionPath: ((_d = this.player.session) === null || _d === void 0 ? void 0 : _d.id) ? 'session.id exists' : 'no session.id'
                });
                this.playerError.emit({
                    message: 'Error saving session',
                    details: error.message || error,
                });
            }
        }
    }
    /**
     * Listens for session changed events and saves updated data for session
     */
    handleSessionChange() {
        this.debouncedSaveSession();
    }
    handlePlayerError(ev) {
        this.playerInfo = null;
        this.errorMessage = ev.detail;
        // Collect as much context as possible for player errors
        const errorContext = {
            errorContext: 'player-error event',
            errorPayload: typeof ev.detail === 'object' ? JSON.stringify(ev.detail) : String(ev.detail),
            playerState: this.player ? 'initialized' : 'not initialized',
            playerConfig: this.player && this.player.config ? 'has config' : 'no config',
            currentRoute: typeof window !== 'undefined' ? window.location.pathname : 'unknown'
        };
        this.trackError('Player error', ev.detail || new Error('Unknown player error'), errorContext);
    }
    /**
     * Helper method to track errors with New Relic
     * This is a utility method that centralizes our error reporting logic and ensures
     * we capture comprehensive error information including stack traces.
     */
    trackError(message, error, customAttributes = {}) {
        var _a, _b, _c;
        // First check if newRelic exists to avoid null reference
        if (!this.newRelic) {
            console.error(`[pie-api-player] ${message}:`, error);
            return;
        }
        const shouldReportToNewRelic = this.newRelic && (_a = this.exceptions, (_a !== null && _a !== void 0 ? _a : false));
        if (shouldReportToNewRelic) {
            // Ensure we have an actual Error object with stack trace
            const errorObj = error instanceof Error ? error : new Error(typeof error === 'string' ? error : JSON.stringify(error));
            // Extract stack trace if available
            const stackTrace = errorObj.stack || '';
            // Add default attributes with comprehensive context
            const attributes = Object.assign(Object.assign({}, customAttributes), { itemId: this.itemId, sessionId: this.sessionId, message, appVersion: version$1.VERSION, errorMessage: errorObj.message || String(error), stackTrace, componentState: {
                    mode: ((_b = this.env) === null || _b === void 0 ? void 0 : _b.mode) || this.initialMode,
                    role: ((_c = this.env) === null || _c === void 0 ? void 0 : _c.role) || this.initialRole,
                    preview: this.preview
                }, time: new Date().toISOString() });
            // Use New Relic's noticeError API to report the error
            this.newRelic.noticeError(errorObj, attributes);
        }
        // Always log to console for debugging
        console.error(`[pie-api-player] ${message}:`, error, customAttributes);
    }
    async updateItemOrSession() {
        try {
            this.playerInfo = null;
            if (!this.preview) {
                await this.load();
            }
            this.el.forceUpdate();
        }
        catch (err) {
            this.trackError('Error updating item or session', err, {
                errorContext: 'updateItemOrSession',
                itemId: this.itemId,
                sessionId: this.sessionId,
                preview: this.preview,
                updateTriggeredBy: 'property change'
            });
            this.playerError.emit({
                message: 'Error updating item or session',
                details: err.message || err,
            });
        }
    }
    readyToLoad() {
        return (this.itemId || this.sessionId) && this.playerClient && this.token;
    }
    getOverrides() {
        let overrides = undefined;
        try {
            if (typeof this.overrides === 'string') {
                overrides = JSON.parse(this.overrides);
            }
            else {
                if (this.overrides) {
                    overrides = this.overrides;
                }
            }
        }
        catch (e) {
        }
        return overrides;
    }
    async load() {
        var _a, _b;
        try {
            if (this.preview) {
                this.env
                    ? (this.env.role = version$1.Role.instructor)
                    : (this.env = { role: version$1.Role.instructor, mode: version$1.Mode.view });
                const buildPlayerConfig = (response) => {
                    var _a;
                    // TODO pie-api-aws 10. test this once passages are tested
                    const passage = (_a = response) === null || _a === void 0 ? void 0 : _a.passage;
                    const out = {
                        session: { id: '', data: [] },
                    };
                    if (passage) {
                        return Object.assign(Object.assign({}, out), { item: { pie: response.config, passage: passage.config } });
                    }
                    else {
                        return Object.assign(Object.assign({}, out), { item: response.config });
                    }
                };
                if (this.itemId && this.token) {
                    const contentData = await version$1.loadFromPieApi(this.token, this.itemId, this.safeConfig.host, this.getOverrides());
                    const info = buildPlayerConfig(contentData);
                    console.log(info);
                    console.log(this);
                    this.modelReload.emit(info);
                    this.playerInfo = info;
                }
                else {
                    this.errorMessage = 'no item id passed to player';
                }
            }
            else {
                if (!this.playerInfo && this.readyToLoad()) {
                    // Track load start in New Relic
                    if (this.newRelic && this.trackPageActions) {
                        this.newRelic.addPageAction('item_load_started', {
                            itemId: this.itemId,
                            sessionId: this.sessionId
                        });
                    }
                    this.playerInfo = await this.playerClient.load(this.itemId, this.sessionId, Object.assign(Object.assign({}, this.env), { mode: this.env && this.env.mode ? this.env.mode : this.initialMode, role: this.env && this.env.role ? this.env.role : this.initialRole }), this.assignmentId, this.getOverrides());
                    // Track successful load in New Relic
                    if (this.newRelic && this.trackPageActions) {
                        this.newRelic.addPageAction('item_loaded', {
                            itemId: this.itemId,
                            sessionId: (_a = this.playerInfo.session) === null || _a === void 0 ? void 0 : _a.id
                        });
                    }
                    let apiInfo = this.playerInfo.item;
                    // is passage model
                    if (apiInfo && apiInfo.pie) {
                        if (this.playerInfo.js && this.playerInfo.js.view) {
                            // deprecated, pie-api should return `bundle.url` prop
                            // we need to create 2 different bundles because for stimulus items we render two pie-player instances PD-4664
                            const [bundle1, bundle2] = version$1.splitStimulusItemsBundle(this.playerInfo.js.view);
                            apiInfo.pie.bundle = { url: bundle1 };
                            apiInfo.passage.bundle = { url: bundle2 };
                        }
                    }
                    else {
                        if (this.playerInfo.js && this.playerInfo.js.view) {
                            apiInfo.bundle = { url: this.playerInfo.js.view };
                        }
                    }
                    this.playerInfo.item = apiInfo;
                    if (!this.sessionId &&
                        this.playerInfo.session &&
                        this.playerInfo.session.id) {
                        this.sessionCreated.emit(this.playerInfo.session);
                    }
                }
            }
        }
        catch (err) {
            this.errorMessage = 'error loading item';
            console.error(err);
            this.trackError('Error loading item', err, {
                errorContext: 'load method',
                itemId: this.itemId,
                sessionId: this.sessionId,
                isPreview: this.preview,
                hasToken: !!this.token,
                hasApiConfig: !!this._apiConfig,
                apiHost: (_b = this.safeConfig) === null || _b === void 0 ? void 0 : _b.host,
                loadState: this.playerInfo ? 'has playerInfo' : 'no playerInfo',
                readyToLoad: this.readyToLoad(),
                initialMode: this.initialMode,
                initialRole: this.initialRole,
                hasAssignmentId: !!this.assignmentId,
                hasOverrides: !!this.getOverrides()
            });
            this.playerError.emit({
                message: 'Error loading item',
                details: err.message || err,
            });
        }
    }
    async componentWillLoad() {
        this.debouncedSaveSession = debounce_1(this.saveSession.bind(this), 100);
        this.newRelic = new version$1.NewRelicInstrumentationService(this.doc);
        this.newRelic.setApplicationVersion(`pie-api-components@${version$1.VERSION}`);
        this.newRelic.setCustomAttribute('componentType', 'pie-api-player');
        if (this.newRelic && this.trackPageActions) {
            this.newRelic.addPageAction('component_loaded', {
                itemId: this.itemId,
                sessionId: this.sessionId
            });
        }
        if (this.recordScreen) {
            this.watchRecordScreen(this.recordScreen, false);
        }
        this.watchHostEnvironment(this.host);
        this.initPlayerClient();
        if (this.readyToLoad()) {
            await this.load();
        }
    }
    disconnectedCallback() {
        // Record component unload if tracking is enabled
        if (this.newRelic && this.trackPageActions) {
            this.newRelic.addPageAction('component_unloaded', {
                itemId: this.itemId,
                sessionId: this.sessionId
            });
        }
    }
    overrideModels(models, customModels) {
        var _a;
        if (!((_a = customModels) === null || _a === void 0 ? void 0 : _a.length)) {
            return models || [];
        }
        return (models || []).map((model) => (Object.assign(Object.assign({}, model), (customModels.find((cm) => cm.id === model.id) || {}))));
    }
    getConfigParsed(item, models, passageModels) {
        var _a, _b;
        // Check if 'item' has the 'pie' structure
        if (item && item.pie) {
            // Structure with 'pie'
            return Object.assign(Object.assign(Object.assign({}, item), { pie: Object.assign(Object.assign({}, item.pie), { models: this.overrideModels(models || item.pie.models, this.customModels), markup: this.customMarkup || item.pie.markup }) }), (item.passage
                ? {
                    passage: Object.assign(Object.assign({}, item.passage), { models: this.overrideModels(passageModels || item.passage.models, this.customPassageModels) }),
                }
                : {}));
        }
        // Standard structure
        return Object.assign(Object.assign({}, item), { models: this.overrideModels(models || ((_a = item) === null || _a === void 0 ? void 0 : _a.models), this.customModels), markup: this.customMarkup || ((_b = item) === null || _b === void 0 ? void 0 : _b.markup) });
    }
    render() {
        const { item } = this.playerInfo || {};
        const configParsed = this.getConfigParsed(item);
        if (this.playerInfo && this.playerInfo.item) {
            return (core.h("pie-player", { class: `pie-api-player ${this.customClassname || ''}`.trim(), hosted: !this.preview, bundleHost: this.host, addCorrectResponse: this.preview, config: configParsed, session: this.playerInfo.session, renderStimulus: this.renderStimulus, allowedResize: this.allowedResize, showBottomBorder: this.showBottomBorder, env: {
                    mode: this.env && this.env.mode ? this.env.mode : this.initialMode,
                    role: this.env && this.env.role ? this.env.role : this.initialRole,
                }, ref: (el) => (this.player = el), bundleEndpoints: this.bundleEndpoints, reFetchBundle: this.reFetchBundle, externalStyleUrls: this.externalStyleUrls, customClassname: this.customClassname }, ' '));
        }
        else {
            if (this.errorMessage) {
                return core.h("error-message", { message: this.errorMessage });
            }
            else {
                return core.h("div", null, "not yet loaded");
            }
        }
    }
    get el() { return core.getElement(this); }
    static get watchers() { return {
        "host": ["watchHostEnvironment"],
        "customConfig": ["onCustomConfigChange"],
        "customModels": ["onCustomModelsChange"],
        "customPassageModels": ["onCustomPassageModelsChange"],
        "customMarkup": ["onCustomMarkupChange"],
        "token": ["watchToken"],
        "sessionId": ["watchItemId"],
        "itemId": ["watchItemId"],
        "maxRetryAttempts": ["watchRetryConfig"],
        "requestTimeoutMs": ["watchRetryConfig"],
        "initialRetryDelayMs": ["watchRetryConfig"],
        "maxRetryDelayMs": ["watchRetryConfig"],
        "trackPageActions": ["watchRetryConfig"],
        "env": ["watchEnv"],
        "recordScreen": ["watchRecordScreen"]
    }; }
    static get style() { return ".pie-api-player img {\n  max-width: inherit;\n  max-height: inherit;\n}\n\n/* These styles are used in the body content of an item -- they also appear in the item body editor. */\n\n.book-title-k5 {\n  text-decoration: underline;\n}\n\n.book-title {\n  font-style: italic;\n}\n\n.block-quote {\n  font-family: serif;\n  margin-left: 20%;\n  margin-right: 20%;\n  margin-top: 5px;\n  margin-bottom: 5px;\n  width: 60%;\n  border: 1px solid black;\n  padding: 2px;\n  display: block;\n}\n\n.text-block {\n  font-family: serif;\n  margin-left: 20%;\n  margin-right: 20%;\n  margin-top: 5px;\n  margin-bottom: 5px;\n  width: 60%;\n  border: 1px solid black;\n  padding: 2px;\n  display: block;\n}\n\n.short-quote {\n  font-weight: bold;\n}\n\nspan.short-quote:before {\n  content: open-quote;\n}\n\nspan.short-quote:after {\n  content: close-quote;\n}\n\n.word-callout {\n  text-decoration: underline;\n}\n\n.relative-emphasis {\n  text-transform: uppercase;\n}\n\n.content-emphasis {\n  font-weight: bold;\n  color: red;\n}\n\nspan.passage-title:before {\n  content: open-quote;\n}\n\nspan.passage-title:after {\n  content: close-quote;\n}\n\n.proper-name {\n  font-style: italic;\n}\n\n.variable {\n  font-style: italic;\n}\n\n.equation-block {\n  text-align: center;\n  margin-left: 20%;\n  margin-right: 20%;\n  margin-top: 5px;\n  margin-bottom: 5px;\n  width: 60%;\n  padding: 2px;\n  display: block;\n}\n\n.embedded-error {\n  font-style: italic;\n}\n\n/* PASSAGES */\n\ndiv.passage-title,\ndiv.passage-subtitle {\n  font-weight: bold;\n  text-align: center;\n}\n\ndiv.passage-author {\n  text-align: center;\n}\n\n.p-number {\n  float: left;\n  font-size: 0.85em;\n}\n\n.numbered-paragraph {\n  margin-left: 36px;\n}\n\n/* ch5680 - also temporary kds content styles that need to be externalized */\n/* Inspect Styles */\n\n\@media print {\n  .kds-noprint {\n    display: none;\n  }\n\n  .noprint {\n    display: none;\n  }\n}\n\n.kds-absolute {\n  border-left: 1px solid black;\n  border-right: 1px solid black;\n  padding: 0px 0.3em 0px 0.3em;\n}\n\n.abs {\n  border-left: #000000 1px solid;\n  border-right: #000000 1px solid;\n  padding: 0px 0.3em 0px 0.3em;\n}\n\n.kds-border-1 {\n  border: 1px solid black;\n}\n\n.kds-border-2 {\n  border: 2px solid black;\n}\n\n.kds-center {\n  margin: 0px;\n  text-align: center;\n}\n\n.center {\n  text-align: center;\n  margin: 0;\n}\n\n.kds-exponent {\n  font-size: 85%;\n  margin-left: -0.4em;\n  position: relative;\n  top: -1.4em;\n}\n\nsup.frac {\n  position: relative;\n  top: -1.4em;\n  font-size: 70%;\n  margin-left: -0.4em;\n}\n\n.kds-fillin {\n  border: 1px solid black;\n  display: inline-block;\n  margin: 0em 0.2em;\n  padding-left: 1em;\n}\n\n.fillin {\n  margin: 0em 0.2em;\n  padding-left: 1em;\n  border: #000000 1px solid;\n  display: inline-block;\n  *display: inline;\n}\n\n.kds-font-larger {\n  font-size: larger;\n}\n\n.kds-font-smaller {\n  font-size: smaller;\n}\n\ntable.kds-fraction {\n  border: 0px;\n  display: inline-block;\n  font-size: 85%;\n  margin: 0px;\n  padding: 0px;\n  vertical-align: middle;\n}\n\ntable.kds-fraction > tbody > tr > td.kds-numerator {\n  border-bottom: 1px solid black;\n  border-left: 0px;\n  border-right: 0px;\n  border-top: 0px;\n  text-align: center;\n}\n\ntable.kds-fraction > tbody > tr > td.kds-denominator {\n  border: 0px;\n  text-align: center;\n}\n\ntable.frac {\n  display: inline-block;\n  *display: inline;\n  vertical-align: middle;\n  font-size: 85%;\n  border: 0px;\n  margin: 0px;\n  padding: 0px;\n}\n\n.frac .nu {\n  border-left: 0px;\n  border-top: 0px;\n  border-right: 0px;\n  border-bottom: #000000 1px solid;\n  text-align: center;\n}\n\n.frac .de {\n  border: 0px;\n  text-align: center;\n}\n\n.kds-hanging-indent {\n  margin: 0px;\n  padding-left: 3em;\n  text-indent: -3em;\n}\n\n.kds-image-caption {\n  font-family: Verdana, Arial, sans-serif;\n  text-align: center;\n}\n\n.kds-image-left {\n  float: left;\n  margin-right: 5px;\n}\n\n.kds-image-right {\n  float: right;\n  margin-left: 5px;\n}\n\n.kds-indent {\n  text-indent: 3em;\n}\n\n.indent {\n  text-indent: 3em;\n}\n\n.kds-nowrap {\n  display: inline-block;\n  white-space: nowrap;\n}\n\n.kds-overline {\n  text-decoration: overline;\n}\n\n.kds-parentheses {\n  font-family: \"Arial Narrow\", Arial, sans-serif;\n  font-size: 300%;\n  font-weight: normal;\n  position: relative;\n  top: 8px;\n}\n\nb.frac {\n  font-weight: normal;\n  font-size: 300%;\n  font-family: arial narrow;\n  position: relative;\n  top: +8px;\n}\n\n.kds-pi {\n  font-family: \"Times New Roman\", serif;\n  font-size: 1.2em;\n}\n\n.newradical {\n  border-collapse: collapse;\n  display: inline-block;\n  *display: inline;\n  vertical-align: middle;\n}\n\n.newradical td {\n  border: 0px;\n  padding: 0px;\n  vertical-align: top;\n}\n\n.newradical td.vinculum {\n  border-top: 1px solid black;\n  padding: 0px;\n  vertical-align: top;\n}\n\n.newradical div.exp {\n  font-size: 0.7em;\n  position: relative;\n  left: 7px;\n  bottom: 5px;\n}\n\n.kds-underline {\n  text-decoration: underline;\n}\n\n.under {\n  text-decoration: underline;\n}\n\n.kds-verdana2t {\n  border: 1px solid white;\n  text-align: left;\n  text-decoration: none;\n  vertical-align: top;\n}\n\n.Verdana2t {\n  vertical-align: top;\n  text-align: left;\n  text-decoration: none;\n  border: 1px solid white;\n}\n\n.kds-whole-number {\n  display: inline-block;\n  font-size: 120%;\n  margin: 0em 0em 0em 0.2em;\n}\n\n.whole {\n  display: inline-block;\n  *display: inline;\n  margin: 0em 0em 0em 0.2em;\n  font-size: 120%;\n}\n\ntable.KdsTable01 {\n  border: 1px solid black;\n  border-collapse: collapse;\n  border-spacing: 0px;\n}\n\ntable.KdsTable01 > tbody > tr > th {\n  background-color: #d3d3d3;\n  border: 1px solid black;\n  font-weight: normal;\n  padding: 2px;\n  text-align: center;\n}\n\ntable.KdsTable01 > tbody > tr > th.bold {\n  background-color: #d3d3d3;\n  border: 1px solid black;\n  font-weight: bold;\n  padding: 2px;\n  text-align: center;\n}\n\ntable.KdsTable01 > tbody > tr > td {\n  border: 1px solid black;\n  padding: 2px;\n  text-align: center;\n}\n\ntable.KdsTable01 > tbody > tr > td.bold {\n  border: 1px solid black;\n  font-weight: bold;\n  padding: 2px;\n  text-align: center;\n}\n\ntable.KdsTable02 {\n  border: 1px solid black;\n  border-collapse: collapse;\n  border-spacing: 0px;\n}\n\ntable.KdsTable02 > tbody > tr > th {\n  background-color: #d3d3d3;\n  border: 1px solid black;\n  font-weight: normal;\n  padding: 2px;\n}\n\ntable.KdsTable02 > tbody > tr > td {\n  border: 1px solid black;\n  padding: 2px;\n}\n\n/* LEARNOSITY OVERRIDES */\n.lrn_feature h3 {\n  margin-top: 0 !important;\n}\n\n/* PIE OVERRIDES */\n\n#stimulus {\n  width: 50%;\n  float: left;\n}\n\n#item {\n  width: 50%;\n  float: left;\n}\n\n/* bootstrap table styles */\n.table {\n  width: 100%;\n  margin-bottom: 1em;\n  background-color: transparent\n}\n\nth {\n  text-align: inherit;\n}\n\n.table td,.table th {\n  padding: .75em;\n  vertical-align: top;\n  border-top: 1px solid #dee2e6\n}\n\n.table thead th {\n  vertical-align: bottom;\n  border-bottom: 2px solid #dee2e6\n}\n\n.table-bordered {\n  border: 1px solid #dee2e6\n}\n\n.table-bordered td,.table-bordered th {\n  border: 1px solid #dee2e6\n}\n\n.table-bordered thead td,.table-bordered thead th {\n  border-bottom-width: 2px\n}"; }
};

exports.pie_api_player = PieApiPlayer;
