'use strict';

var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

function unwrapExports (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

const RE_SCOPED = /^(@[^/]+\/[^/@]+)(?:\/([^@]+))?(?:@([\s\S]+))?/;
const RE_NORMAL = /^([^/@]+)(?:\/([^@]+))?(?:@([\s\S]+))?/;

var parsePackageName = function (input) {
  if (typeof input !== 'string') {
    throw new TypeError('Expected a string')
  }

  const matched = input.charAt(0) === '@' ? input.match(RE_SCOPED) : input.match(RE_NORMAL);

  if (!matched) {
    throw new Error(`[parse-package-name] "${input}" is not a valid string`)
  }

  return {
    name: matched[1],
    path: matched[2] || '',
    version: matched[3] || ''
  }
};

// prefix new tag with pp- for pie player and to ensure custom element validity
const createTag = npmPkg => `pp-${packageToElementName(npmPkg)}`;
/**
 * Replaces all user-defined element name mappings with ones derived from
 * the NPM package name.
 * @param content the content to normalize
 */
const normalizeContentElements = (content) => {
    if (!content || (content && !content.elements))
        return content;
    let markup = content.markup;
    Object.keys(content.elements).forEach(key => {
        const tag = key;
        const npmPkg = content.elements[key];
        const newTag = createTag(npmPkg);
        // markup = markup.split(tag).join(newTag); // - with this approach, not only html tags are replaced, but also the text
        markup = markup
            .split(`<${tag}`)
            .join(`<${newTag}`)
            .split(`</${tag}`)
            .join(`</${newTag}`);
        if (content.models) {
            content.models.forEach(model => {
                if (model.element === key) {
                    model.element = newTag;
                }
            });
        }
        if (key !== newTag) {
            content.elements[newTag] = npmPkg;
            delete content.elements[key];
        }
    });
    content.markup = markup;
    return content;
};
/**
 * Convert an npm package to html valid element name by replacing
 * all special chars with `-`
 * @param npmPackage npm package to convert
 */
const packageToElementName = (npmPackage) => {
    const parsed = parsePackageName(npmPackage);
    if (parsed) {
        let tag = parsed.name.replace(/\/|\./g, "-");
        tag = tag.replace("@", "");
        return tag;
    }
};
const getPackageWithoutVersion = packages => {
    const packagesArray = packages.split("+");
    const newPackageArray = [];
    packagesArray.forEach(p => newPackageArray.push(parsePackageName(p).name));
    return newPackageArray.join("+");
};
/**
 * See if the `PieContent.elements` set contains the provided package.
 *
 * @param elements the elements dict
 * @param npmPackage the npm package to locate
 */
const elementsHasPackage = (elements, npmPackage) => {
    if (elements) {
        const packageToFind = parsePackageName(npmPackage).name;
        return Object.values(elements).find(val => packageToFind === parsePackageName(val).name)
            ? true
            : false;
    }
    else {
        return false;
    }
};
/**
 * Gets all models defined for a given npmPacakge in `PieContent`
 * @param pieContent the pie content
 * @param npmPackage npm package name
 */
const modelsForPackage = (pieContent, npmPackage) => {
    if (pieContent && pieContent.models && pieContent.elements && npmPackage) {
        const element = elementForPackage(pieContent, npmPackage);
        return pieContent.models.filter(m => {
            return element === m.element;
        });
    }
    else {
        return [];
    }
};
/**
 * Gets the element tag defined for a package.
 * @param pieContent the pie content
 * @param npmPackage npm package name
 */
const elementForPackage = (pieContent, npmPackage) => {
    const elems = pieContent && pieContent.elements
        ? Object.keys(pieContent.elements).filter(elTag => {
            const pkg = pieContent.elements[elTag];
            return parsePackageName(npmPackage).name === parsePackageName(pkg).name;
        })
        : [];
    if (elems.length > 1) {
        throw new Error("invalid item format: multiple elements for package.");
    }
    else {
        return elems[0];
    }
};
const getPackageBundleUri = (pies) => {
    return encodeURI(Object.values(pies).join("+"));
};
const pieContentFromConfig = (config) => {
    try {
        if (typeof config == "string") {
            config = JSON.parse(config);
        }
        if (config.pie) {
            const ac = config;
            return normalizeContentElements(ac.pie);
        }
        else if (config.elements) {
            const pc = config;
            return normalizeContentElements(pc);
        }
        else {
            console.warn(`invalid pie data model: ${JSON.stringify(config)}`);
            return null;
        }
    }
    catch (err) {
        console.warn(`invalid pie model: ${JSON.stringify(config)}`);
        return null;
    }
};
const patchMDCSwitchFocus = element => {
    let lock = 0;
    element.addEventListener("mouseup", e => {
        if (lock)
            return;
        let { target } = e;
        do {
            if (target.classList &&
                target.classList.contains("mdc-ripple-upgraded--background-focused")) {
                lock++;
                target.classList.remove("mdc-ripple-upgraded--background-focused");
                lock--;
                return;
            }
        } while ((target = target.parentNode));
    }, { passive: true });
};
/**
 * Creates short id for use within pie models.
 * Can be used to create random element tag or model id.
 */
const pieShortIdGenerator = () => {
    // Not an advanced algorighm, but only need to be unique within the current model.
    var S4 = function () {
        return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1);
    };
    return `p-${S4() + S4()}`;
};
/**
 * Wait for the given milliseconds.
 * @param ms
 */
const waitFor = (ms) => {
    return new Promise(resolve => setTimeout(resolve, ms));
};
/**
 * Execute an async function and retry with exponential backoff
 * based on the maximum retry attempts it can perform.
 *
 * @param fn: Promise-returning function to retry.
 * @param maxRetries: The maximum number of times to attempt the function.
 * @param delay: The delay, in milliseconds.
 */
const withRetry = (fn, maxRetries, delay = 100, maxDelay) => {
    const retryWithBackoff = async (retries) => {
        try {
            let currentDelay = 0;
            // don't wait on the first attempt
            if (retries > 0) {
                // on every retry, exponentially increase the time to wait
                currentDelay = Math.pow(2, retries) * delay;
                if (maxDelay && maxDelay < currentDelay) {
                    currentDelay = maxDelay;
                }
                await waitFor(currentDelay);
            }
            return await fn(currentDelay, retries);
        }
        catch (err) {
            // retry if the limit isn't reached, otherwise throw the error
            if (retries < maxRetries) {
                return retryWithBackoff(retries + 1);
            }
            else {
                console.info("[retryWithBackoff] Max retries reached.");
                throw err;
            }
        }
    };
    return retryWithBackoff(0);
};

exports.commonjsGlobal = commonjsGlobal;
exports.createCommonjsModule = createCommonjsModule;
exports.createTag = createTag;
exports.elementForPackage = elementForPackage;
exports.elementsHasPackage = elementsHasPackage;
exports.getPackageBundleUri = getPackageBundleUri;
exports.getPackageWithoutVersion = getPackageWithoutVersion;
exports.modelsForPackage = modelsForPackage;
exports.normalizeContentElements = normalizeContentElements;
exports.parseNpm = parsePackageName;
exports.patchMDCSwitchFocus = patchMDCSwitchFocus;
exports.pieContentFromConfig = pieContentFromConfig;
exports.pieShortIdGenerator = pieShortIdGenerator;
exports.unwrapExports = unwrapExports;
exports.withRetry = withRetry;
