var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsonHeaders } from "../types";
import { NewRelicEnabledClient } from "./new-relic";
// Default retry configuration values
const DEFAULT_RETRY_CONFIG = {
    maxRetryAttempts: 2,
    requestTimeoutMs: 10000,
    initialRetryDelayMs: 500,
    maxRetryDelayMs: 2000,
    trackPageActions: false
};
/**
 * Shared fetch utility class that provides retry capability
 * Can be used standalone or extended by other clients
 */
export class FetchUtils extends NewRelicEnabledClient {
    constructor(trackingInfo = {}, config) {
        super();
        this.trackingInfo = trackingInfo;
        this.retryConfig = Object.assign(Object.assign({}, DEFAULT_RETRY_CONFIG), config);
    }
    /**
     * Implementation of the abstract method from NewRelicEnabledClient
     */
    getTrackingBaseAttributes() {
        return Object.assign(Object.assign({}, this.trackingInfo), { networkInfo: this.getNetworkInfo() });
    }
    /**
     * Create a sanitized sample of request data that's safe to log
     */
    createSanitizedDataSample(data) {
        if (!data || typeof data !== 'object')
            return {};
        const result = {};
        Object.keys(data).forEach(key => {
            const value = data[key];
            result[key] = typeof value === 'string'
                ? (value.length > 50 ? value.substring(0, 50) + '...' : value)
                : typeof value;
        });
        return result;
    }
    /**
     * Check if an error is a network error
     */
    isNetworkError(error) {
        return (error.name === "AbortError" ||
            error instanceof TypeError ||
            error.message.includes("NetworkError") ||
            error.message.includes("Failed to fetch") ||
            error.message.includes("Network request failed") ||
            !(typeof navigator !== 'undefined' && navigator.onLine));
    }
    /**
     * Track API call start if trackPageActions is enabled
     */
    trackOperationStart(endpointName, operationType = 'request', dataSample = {}) {
        if (!this.retryConfig.trackPageActions || !this.newRelic)
            return;
        this.track('activity', `api_${operationType}_started`, {
            endpoint: endpointName,
            dataSample: JSON.stringify(dataSample),
            timestamp: new Date().toISOString()
        });
    }
    /**
     * Track API call complete if trackPageActions is enabled
     */
    trackOperationComplete(operationType, endpointName, startTime) {
        if (this.retryConfig.trackPageActions && operationType !== 'request') {
            this.track('activity', `api_${operationType}_complete`, {
                endpoint: endpointName,
                duration: Date.now() - startTime,
                success: true
            });
        }
    }
    /**
     * Track API call completion
     */
    trackRequestComplete(endpointName, attempt, response, duration, responseData) {
        // Always track retries and failures
        if (attempt > 0 || !response.ok) {
            // This will track retry attempts (success or failure) and all failures
            this.track('activity', "api_request_complete", {
                apiEndpoint: endpointName,
                attempt,
                status: response.status,
                duration,
                success: response.ok,
                outcome: response.ok
                    ? "retry_success" // We know attempt > 0 here
                    : (attempt >= this.retryConfig.maxRetryAttempts ? "non_recoverable_failure" : "failure_will_retry")
            });
        }
        // Track successful first attempts when trackPageActions is enabled
        else if (this.retryConfig.trackPageActions) {
            // This is a first attempt success (attempt === 0 && response.ok)
            const responseSample = responseData
                ? this.createSanitizedDataSample(responseData)
                : {};
            this.track('activity', "api_request_complete", {
                endpoint: endpointName,
                status: response.status,
                duration,
                outcome: "first_attempt_success",
                responseSample: JSON.stringify(responseSample)
            });
        }
    }
    /**
     * Track retry attempt
     */
    trackRetryAttempt(endpointName, attempt, dataSample) {
        this.track('activity', "api_retry_attempt", Object.assign({ endpoint: endpointName, attempt }, (dataSample && { dataSample: JSON.stringify(dataSample) })));
    }
    /**
     * Track error
     */
    trackError(error, attributes, responseInfo) {
        const errorType = responseInfo ? "api_error" : "api_final_error";
        const { finalAttempt, isNetworkError } = attributes, customAttributes = __rest(attributes
        /**
         * Transient (retryable) errors:
         * - 400 Request Timeout
         * - 429 Too Many Requests
         * - 500–502, 503, 504 Server Errors (sometimes recoverable)
         * - Network errors
         */
        , ["finalAttempt", "isNetworkError"]);
        /**
         * Transient (retryable) errors:
         * - 400 Request Timeout
         * - 429 Too Many Requests
         * - 500–502, 503, 504 Server Errors (sometimes recoverable)
         * - Network errors
         */
        const recoveryStatus = responseInfo && [408, 429, 500, 502, 503, 504].includes(responseInfo.status) || !!isNetworkError
            ? 'transient_recoverable'
            : 'non_recoverable';
        const errorData = Object.assign(Object.assign({}, customAttributes), { finalAttempt: !!finalAttempt, errorName: error.name, errorMessage: error.message, errorType,
            recoveryStatus, stackTrace: error.stack });
        if (responseInfo) {
            Object.assign(errorData, Object.assign({ status: responseInfo.status, statusText: responseInfo.statusText }, (responseInfo.responseData && { responseData: responseInfo.responseData })));
        }
        else {
            Object.assign(errorData, {
                isNetworkError: !!isNetworkError,
            });
        }
        if (recoveryStatus === 'transient_recoverable') {
            this.track('activity', 'transient_error', errorData, error);
        }
        else {
            this.track('error', errorType, errorData, error);
        }
    }
    /**
     * Calculate exponential backoff wait time for retries
     */
    calculateBackoffTime(attempt) {
        return Math.min(this.retryConfig.initialRetryDelayMs * Math.pow(2, attempt), this.retryConfig.maxRetryDelayMs);
    }
    /**
     * Wait for network connectivity if offline
     */
    async waitForConnectivity(endpointName, attempt) {
        if (typeof navigator !== 'undefined' && !navigator.onLine && typeof window !== 'undefined') {
            this.track('activity', "api_offline_retry", {
                endpoint: endpointName,
                attempt,
                waitingForConnectivity: true
            });
            await new Promise((resolve) => {
                const onlineHandler = () => {
                    window.removeEventListener('online', onlineHandler);
                    resolve();
                };
                setTimeout(() => {
                    window.removeEventListener('online', onlineHandler);
                    resolve();
                }, this.retryConfig.maxRetryDelayMs);
                window.addEventListener('online', onlineHandler);
            });
            return;
        }
        // Apply standard backoff
        const waitTime = this.calculateBackoffTime(attempt);
        console.warn(`Retrying ${endpointName} (attempt ${attempt + 1}/${this.retryConfig.maxRetryAttempts + 1}) after ${waitTime}ms...`);
        this.track('activity', "api_retry_scheduled", {
            endpoint: endpointName,
            attempt,
            waitTime
        });
        await new Promise(resolve => setTimeout(resolve, waitTime));
    }
    /**
     * Core retry logic for any fetch operation
     */
    async executeWithRetry(operation, endpointName, operationType = 'request', dataSample) {
        const startTime = Date.now();
        // Track start if enabled
        this.trackOperationStart(endpointName, operationType, dataSample);
        for (let attempt = 0; attempt <= this.retryConfig.maxRetryAttempts; attempt++) {
            try {
                // Track retry attempts
                if (attempt > 0) {
                    this.trackRetryAttempt(endpointName, attempt, dataSample);
                }
                // Execute the operation
                const result = await operation(attempt);
                // If we're here, the operation succeeded
                this.trackOperationComplete(operationType, endpointName, startTime);
                return result;
            }
            catch (error) {
                const typedError = error instanceof Error ? error : new Error(String(error));
                const isNetworkError = this.isNetworkError(typedError);
                // If it's the last attempt or not a network error, throw
                if (attempt >= this.retryConfig.maxRetryAttempts || !isNetworkError) {
                    this.trackError(typedError, {
                        endpoint: endpointName,
                        attempt,
                        duration: Date.now() - startTime,
                        finalAttempt: attempt === this.retryConfig.maxRetryAttempts,
                        isNetworkError
                    });
                    throw typedError;
                }
                // Wait before retrying
                await this.waitForConnectivity(endpointName, attempt);
            }
        }
        // Should never reach here
        throw new Error(`Operation ${endpointName} failed after ${this.retryConfig.maxRetryAttempts + 1} attempts.`);
    }
    /**
     * Make an API call with configurable retry logic
     *
     * @param url - Full URL to fetch
     * @param options - Request options (method, headers, body, etc.)
     * @param endpointName - Name of endpoint for tracking purposes
     * @returns A promise that resolves to the parsed response data
     */
    async fetchWithRetry(url, options, endpointName) {
        let timeoutId;
        let controller;
        const hasAbortController = typeof AbortController !== 'undefined';
        // Extract request body for logging if present
        const requestBody = options.body ?
            (typeof options.body === 'string' ? JSON.parse(options.body) : options.body) :
            undefined;
        const dataSample = this.createSanitizedDataSample(requestBody);
        return this.executeWithRetry(async (attempt) => {
            var _a;
            const startTime = Date.now();
            // Clear any previous timeout
            if (timeoutId) {
                clearTimeout(timeoutId);
                timeoutId = undefined;
            }
            // Set up abort controller for timeouts (if available)
            if (hasAbortController) {
                controller = new AbortController();
                timeoutId = window.setTimeout(() => {
                    var _a;
                    (_a = controller) === null || _a === void 0 ? void 0 : _a.abort();
                }, this.retryConfig.requestTimeoutMs);
                // Add signal to options
                options.signal = controller.signal;
            }
            try {
                // Execute the fetch
                const response = await fetch(url, options);
                // Clear timeout after response
                if (timeoutId) {
                    clearTimeout(timeoutId);
                    timeoutId = undefined;
                }
                // Process response
                let responseData = {};
                let responseText = '';
                if (response.status !== 204) {
                    try {
                        // In test environments, response might not have clone method
                        if (typeof response.clone === 'function') {
                            responseText = await response.clone().text();
                        }
                        else {
                            // For test environments where response.clone is not available
                            try {
                                // Try to get text directly if possible
                                responseText = await response.text();
                            }
                            catch (e) {
                                // If that fails too, try to access responseText directly (for mock responses)
                                responseText = response.bodyUsed ? '' : response.body || '';
                            }
                        }
                        if (responseText && responseText.trim()) {
                            try {
                                responseData = JSON.parse(responseText);
                            }
                            catch (e) {
                                responseData = { error: `Parse error: ${e.message}`, rawResponse: responseText };
                            }
                        }
                    }
                    catch (e) {
                        console.warn(`Failed to parse response as JSON: ${e}`);
                    }
                }
                this.trackRequestComplete(endpointName, attempt, response, Date.now() - startTime, responseData);
                if (response.ok) {
                    return responseData;
                }
                // Handle error response
                const error = new Error(`Call to ${endpointName} failed. ${((_a = responseData) === null || _a === void 0 ? void 0 : _a.error) || 'HTTP ' + response.status}`);
                if (attempt >= this.retryConfig.maxRetryAttempts) {
                    this.trackError(error, {
                        endpoint: endpointName,
                        attempt,
                        duration: Date.now() - startTime,
                        finalAttempt: true,
                        isNetworkError: false
                    }, {
                        status: response.status,
                        statusText: response.statusText,
                        responseData: typeof responseData === 'object' ? JSON.stringify(responseData) : responseText
                    });
                }
                throw error;
            }
            finally {
                // Clear any timeout
                if (timeoutId) {
                    clearTimeout(timeoutId);
                    timeoutId = undefined;
                }
            }
        }, endpointName, 'request', dataSample);
    }
    /**
     * Convenience method for API calls with standard JSON headers and authorization
     */
    async callApi(url, method, token, data, customHeaders) {
        const headers = Object.assign(Object.assign(Object.assign({}, jsonHeaders()), { "Authorization": `Bearer ${token}`, "x-date": new Date().getTime().toString(), "x-request-id": `${Date.now()}` }), customHeaders);
        const fetchOptions = {
            method: method.toUpperCase(),
            mode: "cors",
            headers,
            body: data !== undefined ? JSON.stringify(data) : undefined,
        };
        return this.fetchWithRetry(url, fetchOptions, `${method.toUpperCase()} ${url}`);
    }
    /**
     * Convenience method for GraphQL requests
     */
    async executeGraphQL(endpoint, token, query, variables) {
        return this.callApi(endpoint, 'POST', token, {
            query,
            variables
        }, {
            "Accept-Encoding": "gzip, deflate, br",
            "Accept": "application/json"
        });
    }
    /**
     * Convenience method for file uploads with direct signed URLs
     */
    async uploadFile(signedURL, file, endpointName = "file-upload") {
        const options = {
            method: "PUT",
            body: file,
            headers: {
                "Content-Type": file.type,
                "Content-Length": file.size.toString()
            }
        };
        // For file uploads, we want the raw response object, not parsed JSON
        return this.executeWithRetry(async () => {
            const response = await fetch(signedURL, options);
            if (response.ok) {
                return response;
            }
            // Handle error response
            throw new Error(`Upload to ${endpointName} failed. HTTP ${response.status}`);
        }, endpointName, 'upload');
    }
    /**
     * Convenience method for DELETE operations with signed URLs
     */
    async deleteWithSignedUrl(signedURL, endpointName = "resource-delete") {
        const options = {
            method: "DELETE"
        };
        // For DELETE operations, we want the raw response object
        return this.executeWithRetry(async () => {
            const response = await fetch(signedURL, options);
            if (response.ok) {
                return response;
            }
            // Handle error response
            throw new Error(`Delete operation to ${endpointName} failed. HTTP ${response.status}`);
        }, endpointName, 'delete');
    }
}
