import { HOST_DEFAULTS } from "../defaults";
import { getVersionedId, versionedIdtoString, isPreRelease } from "../util";
import { FetchUtils } from "./fetch-utils";
const log = (..._args) => ({}); //console.log.bind(console, "[graphql]");
// Create a single shared instance of FetchUtils for GraphQL operations
const fetchUtils = new FetchUtils({ source: "graphql-client" });
const removeVersionFromId = (id) => {
    const idParts = id.split('@');
    return idParts[0];
};
export const loadBestVersions = async (token, apiUrl) => {
    if (!token) {
        console.log("no token - skip");
        return;
    }
    const query = `query {
      findAllBestVersions {
        pie
        version
      }
    }`;
    log("loadBestVersions...", token, apiUrl);
    const graphqlEndpoint = apiUrl + "/graphql";
    try {
        const result = await fetchUtils.executeGraphQL(graphqlEndpoint, token, query);
        log("loadBestVersions done", result.errors);
        if (result.errors) {
            console.log("ERRORS:", result.errors);
            throw new Error(result.errors[0].message);
        }
        log("result.data:", result.data);
        log("loadBestVersions done", result.data.findAllBestVersions);
        return result.data.findAllBestVersions;
    }
    catch (err) {
        console.log(err);
        throw new Error(`error with server request ${query}`);
    }
};
export const saveContentItem = async (token, versionedID, releaseType, config, collections = [], apiUrl = HOST_DEFAULTS.prod.apiUrl) => {
    versionedID = versionedID ? versionedID : null;
    const query = `
    mutation SaveItem($input: SaveItemInput!) {
      saveItem(input: $input) {
        id
        name
        retired
        published
        searchMetaData
        passage {
          id
          name
          version {
            major
            minor
            patch
            prerelease
          }
        }
        version {
          major
          minor
          patch
          prerelease
        }
        createdAt
        updatedAt
      }
    }
  `;
    const itemInput = {
        config,
        searchMetaData: {},
    };
    if (versionedID) {
        // Do not send id with version
        itemInput['id'] = removeVersionFromId(versionedID);
    }
    const variables = {
        input: {
            item: itemInput,
            releaseType // (optional release type, defaults to ReleaseType.PRERELEASE)
        },
    };
    if (collections) {
        variables.input['collectionIds'] = collections; // (optional collection IDs)
    }
    const graphqlEndpoint = apiUrl + "/graphql";
    try {
        const saveResult = await fetchUtils.executeGraphQL(graphqlEndpoint, token, query, variables);
        if (saveResult && saveResult.data.saveItem.id) {
            return versionedIdtoString({
                id: saveResult.data.saveItem.id,
                version: saveResult.data.saveItem.version,
            });
        }
    }
    catch (err) {
        console.log(err);
        throw new Error(`error with server request ${query}`);
    }
};
export const releaseContentItem = async (token, versionedID, apiUrl = HOST_DEFAULTS.prod.apiUrl) => {
    const vId = getVersionedId(versionedID);
    const graphqlEndpoint = apiUrl + "/graphql";
    const query = `
    mutation ReleaseItem($id: VersionedID!) {
      releaseItem(vId: $id) {
        id
        version {
          major
          minor
          patch
          prerelease
        }
      }
    }
  `;
    if (isPreRelease(vId)) {
        try {
            const res = await fetchUtils.executeGraphQL(graphqlEndpoint, token, query, {
                id: versionedID
            });
            if (res &&
                res.data &&
                res.data.releaseItem &&
                res.data.releaseItem.id) {
                return versionedIdtoString({
                    id: res.data.releaseItem.id,
                    version: res.data.releaseItem.version,
                });
            }
        }
        catch (err) {
            console.log(err);
            throw new Error(`error with server request ${query}`);
        }
    }
    else {
        return Promise.reject("id does not have a pre-release version");
    }
};
export const loadFromPieApi = async (token, versionedID, apiUrl = HOST_DEFAULTS.prod.apiUrl, overrides) => {
    const graphqlEndpoint = apiUrl + "/graphql";
    const query = `
    query Item($id: VersionedID!, $opts: ItemLoadOptions) {
      item(vId: $id, opts: $opts) {
        id
        name
        config
        searchMetaData
        version {
          major
          minor
          patch
          prerelease
        }
        passage {
          id
          name
          config
          searchMetaData
          version {
            major
            minor
            patch
            prerelease
          }
        }
      }
    }
  `;
    try {
        const res = await fetchUtils.executeGraphQL(graphqlEndpoint, token, query, {
            id: versionedID,
            opts: {
                overrides
            }
        });
        return res && res.data ? res.data.item : null;
    }
    catch (err) {
        console.log(err);
        throw new Error(`error with server request ${query}`);
    }
};
