import { NewRelicEnabledClient } from "./new-relic";
import { FetchUtils } from "./fetch-utils";
const isMethodAndPath = (m) => m && m.path && m.method;
export const toMethodAndPath = (m) => {
    if (isMethodAndPath(m)) {
        return m;
    }
    if (typeof m === "string") {
        return { method: "POST", path: m };
    }
    throw new Error(`Unknown type ${m}`);
};
export class PlayerApiClient extends NewRelicEnabledClient {
    constructor(token, endpoints, optimize, retryConfig) {
        super();
        this.token = token;
        this.endpoints = endpoints;
        this.optimize = optimize;
        // Create a FetchUtils instance with tracking info
        this.fetchUtils = new FetchUtils({
            host: this.endpoints.host,
            optimize: this.optimize,
            hasToken: !!this.token
        }, retryConfig);
    }
    /**
     * Implementation of the abstract method from NewRelicEnabledClient
     */
    getTrackingBaseAttributes() {
        return {
            host: this.endpoints.host,
            optimize: this.optimize,
            hasToken: !!this.token,
            networkInfo: this.getNetworkInfo()
        };
    }
    /**
     * Make an API call with retry mechanism
     *
     * @param endpoint - The API endpoint to call (method and path)
     * @param data - The data to send in the request body (will be JSON stringified)
     * @returns A promise that resolves to the parsed response data
     * @throws Error if all retry attempts fail
     */
    async call(endpoint, data) {
        return this.fetchUtils.callApi(`${this.endpoints.host}${endpoint.path}`, endpoint.method, this.token, data);
    }
    load(itemId, sessionId, env, assignmentId, overrides) {
        const params = {
            itemId,
            sessionId,
            assignmentId,
            env
        };
        if (overrides) {
            params.overrides = overrides;
        }
        return this.call(toMethodAndPath(this.endpoints.load), params);
    }
    saveSession(session, env) {
        return this.call(toMethodAndPath(this.endpoints.saveSession), {
            data: session.data,
            sessionId: session.id,
            env
        });
    }
    model(session, env, overrides) {
        const params = {
            sessionId: session.id,
            data: session.data,
            env
        };
        if (overrides) {
            params.overrides = overrides;
        }
        return this.call(toMethodAndPath(this.endpoints.model), params);
    }
    score(session, env, overrides) {
        const params = {
            sessionId: session.id,
            data: session.data,
            env,
            overrides
        };
        if (overrides) {
            params.overrides = overrides;
        }
        return this.call(toMethodAndPath(this.endpoints.score), params);
    }
}
