import get from "lodash/get";
import { NewRelicEnabledClient } from "./new-relic";
import { FetchUtils } from "./fetch-utils";
export class PieApiSoundSupport extends NewRelicEnabledClient {
    constructor(token, graphql, versionedID, versionedIDGenerator) {
        super();
        this.token = token;
        this.graphql = graphql;
        this.versionedID = versionedID;
        this.versionedIDGenerator = versionedIDGenerator;
        // Create a FetchUtils instance with tracking info
        this.fetchUtils = new FetchUtils({
            graphqlEndpoint: this.graphql,
            hasToken: !!this.token,
            hasVersionedID: !!this.versionedID
        });
    }
    getTrackingBaseAttributes() {
        return {
            graphqlEndpoint: this.graphql,
            hasToken: !!this.token,
            hasVersionedID: !!this.versionedID,
            networkInfo: this.getNetworkInfo()
        };
    }
    async insert(file, done) {
        var _a, _b;
        try {
            const typeParsed = file.type.replace('x-', '');
            const info = await this.createUploadRequest(typeParsed);
            const uploadResponse = await this.fetchUtils.uploadFile(info.signedURL, file, "sound-upload");
            if (uploadResponse.ok) {
                done(null, info.publicURL);
            }
            else {
                const error = new Error(`Upload failed (url ${info.signedURL})`);
                if ((_a = this.newRelic) === null || _a === void 0 ? void 0 : _a.isAvailable()) {
                    this.newRelic.noticeError(error, { signedUrl: info.signedURL, status: uploadResponse.status });
                }
                done(error, null);
            }
        }
        catch (e) {
            const error = new Error(`Upload failed: ${e.message}`);
            if ((_b = this.newRelic) === null || _b === void 0 ? void 0 : _b.isAvailable()) {
                this.newRelic.noticeError(error, { message: e.message });
            }
            done(error, null);
        }
    }
    async createUploadRequest(contentType) {
        var _a;
        // The new pie api made media ID required for performance (previously ID was optional).
        // This means we have to treat this edge case:
        //  If the item is newly created and doesn't have an ID yet (was not saved in the db yet),
        //  we need to generate the ID, upload the media with that ID and also make sure to use the
        //  generated ID in pie-api-author for other item-related actions (eg: upload other medias, save item)
        if (!this.versionedID && this.versionedIDGenerator) {
            this.versionedID = this.versionedIDGenerator();
        }
        const query = `
      mutation RequestMediaUpload($id: ID!, $contentType: String!) {
        requestMediaUpload(id: $id, contentType: $contentType) {
        signedURL
        publicURL
      }
    }`;
        const response = await this.fetchUtils.executeGraphQL(this.graphql, this.token, query, {
            id: this.versionedID,
            contentType
        });
        const data = get(response, "data.requestMediaUpload");
        if (!data) {
            const errors = get(response, "errors");
            const uploadError = errors.find(e => e.path.includes("requestMediaUpload"));
            if ((_a = this.newRelic) === null || _a === void 0 ? void 0 : _a.isAvailable()) {
                this.newRelic.noticeError(uploadError, { errors: errors });
            }
            throw uploadError
                ? new Error(uploadError.message)
                : new Error(`Unknown gql error: ${JSON.stringify(response)}`);
        }
        return data;
    }
    async delete(src, done) {
        var _a, _b;
        if (!src) {
            return;
        }
        const mut = `
      mutation RequestMediaDelete($publicURL: String!) {
        requestMediaDelete(publicURL: $publicURL) {
        signedURL
      }
    }`;
        try {
            const response = await this.fetchUtils.executeGraphQL(this.graphql, this.token, mut, { publicURL: src });
            const info = get(response, "data.requestMediaDelete");
            if (!info) {
                const errors = get(response, "errors");
                const deleteError = (errors || []).find(e => e.path.includes("requestMediaDelete"));
                let error = new Error(deleteError ? deleteError.message : "no info in json?");
                if ((_a = this.newRelic) === null || _a === void 0 ? void 0 : _a.isAvailable()) {
                    this.newRelic.noticeError(error, { errors: errors });
                }
                throw error;
            }
            // Use the specialized delete method for DELETE operations
            const deleteResponse = await this.fetchUtils.deleteWithSignedUrl(info.signedURL, "sound-delete");
            if (deleteResponse.ok) {
                done();
            }
            else {
                let error = new Error("DELETE failed");
                if ((_b = this.newRelic) === null || _b === void 0 ? void 0 : _b.isAvailable()) {
                    this.newRelic.noticeError(error, {});
                }
                throw error;
            }
        }
        catch (e) {
            done(e);
        }
    }
}
