import { h } from "@stencil/core";
import { v4 as uuidv4 } from "uuid";
import { loadFromPieApi, saveContentItem, releaseContentItem, } from "../../clients/graphql";
import { versionedIdtoString, idInfo, lockConfigVersions, unlockConfigVersions, } from "../../util";
import { VERSION } from "../../version";
import { addPackageToContent, addRubric, } from "@pie-framework/pie-player-components/dist/collection/rubric-utils";
import { pieContentFromConfig } from "@pie-framework/pie-player-components/dist/collection/utils/utils";
import { PieApiImageSupport } from "../../clients/image-support";
import { PieApiSoundSupport } from "../../clients/sound-support";
import { HOST_DEFAULTS, PLAYER_HOST_CONFIG } from "../../defaults";
import { get } from "../../pie-versions";
import { NewRelicInstrumentationService } from "../../clients/new-relic";
export class PieApiAuthor {
    constructor() {
        /**
         * Choose 'dev' , 'stage' or 'prod' environments for back end.
         */
        this.host = "prod";
        /**
         * used to automatically re-fetch the bundle (in case we get a 503); defaults to false
         */
        this.reFetchBundle = false;
        /**
         * For development purposes only - allows alternate api endpoints to be provided.
         */
        this.apiConfig = PLAYER_HOST_CONFIG;
        /**
         * Adds a preview view which will render the content in another tab as it may appear to a student or instructor.
         */
        this.addPreview = false;
        /**
         * Enable screen recording with logrocket.
         * Note: not to be used in production unless for temporary use to track down a specific user problem.
         */
        this.recordScreen = false;
        /**
         * Enable exception tracking.
         */
        this.exceptions = false;
        /**
         * If true, the `@version` tag for elements in the configuration will be ignored and a static version for that
         * interaction type determined by the current release of this pie-api-author component will be used.
         */
        this.lockVersions = false;
        this.generateUUID = () => {
            // This function is a callback sent to PieApiImageSupport and PieApiSoundSupport, to treat a specific edge case:
            //  If the item is newly created and doesn't have an ID yet (was not saved in the db yet),
            //  we need to generate the ID, upload the media with that ID and also make sure to use the
            //  generated ID in pie-api-author for other item-related actions (eg: upload other medias, save item)
            this.contentId = uuidv4();
            // Store the generated id
            return this.contentId;
        };
    }
    async watchHostEnvironment(newVal) {
        if (newVal) {
            this.apiConfig.host = HOST_DEFAULTS[newVal].apiUrl;
            this.apiConfig.bundleBaseUrl = HOST_DEFAULTS[newVal].bundleBaseUrl;
            if (this.token) {
                await get(this.token, this.apiConfig.host);
            }
        }
    }
    /**
     * Utility method to add a `@pie-element/rubric` section to an item config when creating an item should be used before setting the config.
     *
     * @deprecated this method is for temporary use, will be removed at next major release
     *
     * @param config the item config to mutate
     * @param rubricModel optional model to pass to `@pie-element/rubric`
     */
    async addRubricToConfig(config, rubricModel) {
        console.warn("This function is deprecated. Use complex-rubric functionality instead.");
        if (!rubricModel) {
            rubricModel = {
                id: "rubric",
                element: "pie-rubric",
                points: ["", "", "", ""],
                excludeZero: false,
            };
        }
        const configPieContent = pieContentFromConfig(config);
        addPackageToContent(configPieContent, "@pie-element/rubric", rubricModel);
        const updatedConfig = addRubric(configPieContent);
        return updatedConfig;
    }
    getOverrides() {
        let overrides = undefined;
        try {
            if (typeof this.overrides === "string") {
                overrides = JSON.parse(this.overrides);
            }
            else {
                if (this.overrides) {
                    overrides = this.overrides;
                }
            }
        }
        catch (e) { }
        return overrides;
    }
    /**
     * Loads a content item from `pie-api` host into the view.
     *
     * @param contentId the unique versioned identifer `id@version` for the content item to load. If this param
     * is not provided and `content-id` attribute has a value, that value will be used to load the content item.
     */
    async load(contentId) {
        if (!this.token) {
            return;
        }
        const idToLoad = contentId ? contentId : this.contentId;
        if (idToLoad) {
            const contentData = await loadFromPieApi(this.token, idToLoad, this.apiConfig.host, this.getOverrides());
            this.contentConfig = contentData.config;
            const stringVersionId = versionedIdtoString({
                id: contentData.id,
                version: contentData.version,
            });
            this.contentId = stringVersionId;
            this.contentLoaded.emit({
                id: this.contentId,
                config: this.contentConfig,
            });
        }
    }
    /**
     * Creates a new content item from the currently loaded contents.
     *
     * @param preReleaseType - see preRelaseType for the `save` method
     */
    async create(preReleaseType = "premajor") {
        const cc = this.shouldLock()
            ? lockConfigVersions(this.contentConfig, this.versions, {})
            : unlockConfigVersions(this.contentConfig);
        return saveContentItem(this.token, this.contentId || null, preReleaseType, cc, this.collectionId ? [this.collectionId] : null, this.apiConfig.host);
    }
    /**
     * Saves an update to an existing pre-released content item.
     * The content item version must be in pre-release state.
     *
     * the `preReleaseType` param determines what kind of version update, if any, will be performed.
     *
     * `null` - (default) writes over existing pre-release content, , e.g.`1.0.0-draft.0 -> 1.0.0-draft.0`\n
     * `prerelease` - writes in increment to the prerelease version, e.g. `1.0.0-draft.0 -> 1.0.0-draft.1`\n
     * `premajor` - creates a new major release from version, e.g. `1.0.0-draft.1 -> 2.0.0-draft.0`\n
     * `preminor` - creates a new minor release from version, e.g. `1.0.0-draft.1 -> 1.1.0-draft.0`\n
     * `prepatch` - creates a new patch release from version, e.g. `1.0.0-draft.1 -> 1.0.1-draft.0`\m
     *
     * @param preReleaseType - the type of update to perform
     */
    async save(preReleaseType = null) {
        let newVersionedId;
        if (this.contentConfig) {
            const cc = this.shouldLock()
                ? lockConfigVersions(this.contentConfig, this.versions, {})
                : unlockConfigVersions(this.contentConfig);
            newVersionedId = await saveContentItem(this.token, this.contentId, preReleaseType, cc, this.collectionId ? [this.collectionId] : null, this.apiConfig.host);
            if (newVersionedId) {
                this.contentId = newVersionedId;
                return newVersionedId;
            }
            else {
                return Promise.resolve(null);
            }
        }
        else {
            return Promise.resolve(null);
        }
    }
    version() {
        return Promise.resolve(VERSION);
    }
    /**
     * this method is for having the possibility to access validateModels from player <br/><br/> It returns the model of the item and errors field which contains the errors returned by validate function in controller. It is an asynchronous method, so it needs to be called with await
     *
     */
    validate() {
        if (this.pieConfigEl) {
            return this.pieConfigEl.validateModels();
        }
        return null;
    }
    /**
     * Release a pre-release version of the content currently loaded in the authoring view.
     *
     * Throws an exception if the version of the current Content Item does not have a pre-release tag e.g.`1.0.1-draft`
     *
     *  example:
     * (`contentId = 'abc@1.0.1-draft`)
     *
     *  `this.release() => 'abc@1.0.1`'
     *
     */
    async release() {
        return releaseContentItem(this.token, this.contentId, this.apiConfig.host);
    }
    /**
     * TODO
     * Gets a list of all versions that exist for this Content Item.
     */
    getVersions() {
        return Promise.resolve([""]);
    }
    async onTokenChange() {
        if (this.token) {
            await get(this.token, this.apiConfig.host);
        }
        this.load(this.contentId);
    }
    async watchContentId(newVal) {
        const info = idInfo(newVal);
        this.contentIdValidated.emit({ valid: !!info, status: info });
    }
    shouldLock() {
        return !this.overrides && this.lockVersions;
    }
    watchRecordScreen(newVal, oldVal) {
        if (newVal && newVal !== oldVal) {
            if (this.newRelic && this.recordScreenIdentity) {
                // Set user identity for New Relic session
                this.newRelic.setUserId(this.recordScreenIdentity);
                this.recordingStarted.emit("Session recording started with New Relic");
            }
        }
    }
    getImageSupport() {
        return new PieApiImageSupport(this.token, this.apiConfig.host + "/graphql", this.contentId, this.generateUUID);
    }
    getUploadSoundSupport() {
        return new PieApiSoundSupport(this.token, this.apiConfig.host + "/graphql", this.contentId, this.generateUUID);
    }
    async componentWillLoad() {
        if (this.token) {
            get(this.token, this.apiConfig.host);
        }
        this.newRelic = new NewRelicInstrumentationService(this.doc);
        this.newRelic.setApplicationVersion(`pie-api-components@${VERSION}`);
        this.newRelic.setCustomAttribute('componentType', 'pie-api-author');
        if (this.recordScreen) {
            this.watchRecordScreen(this.recordScreen, false);
        }
        this.watchHostEnvironment(this.host);
        this.versions = await get(this.token, this.apiConfig.host);
    }
    connectedCallback() {
        /**
         * Some stenciljs bug w/ this binding in event listeners, need to bind it correctly.
         */
        this.onModelUpdated = ((e) => {
            console.log("! this:", this);
            const unlocked = unlockConfigVersions(e.detail);
            this.contentConfig = unlocked;
        }).bind(this);
        if (!this.versions) {
            get(this.token, this.apiConfig.host).then((v) => (this.versions = v));
        }
    }
    componentDidLoad() {
        if (this.pieConfigEl) {
            this.pieConfigEl.VERSION = VERSION;
        }
    }
    getConfigParsed(sanctioned) {
        let configSettings = this.configSettings;
        if (this.configSettings) {
            const elName = Object.keys(this.configSettings)[0];
            configSettings = Object.assign(Object.assign({}, this.configSettings), { [elName]: Object.assign({ pieApi: {
                        token: this.token,
                        host: this.apiConfig.host,
                    } }, this.configSettings[elName]) });
        }
        else if (sanctioned.elements) {
            const value = Object.keys(sanctioned.elements).map((key) => sanctioned.elements[key])[0];
            const elName = `@${value.split("@")[1]}`;
            configSettings = {
                [elName]: {
                    pieApi: {
                        token: this.token,
                        host: this.apiConfig.host,
                    },
                },
            };
        }
        return configSettings;
    }
    render() {
        if (this.contentConfig && this.versions) {
            const is = this.getImageSupport();
            const us = this.getUploadSoundSupport();
            const sanctioned = lockConfigVersions(this.contentConfig, this.versions, this.getOverrides());
            const configSettings = this.getConfigParsed(sanctioned);
            return (h("pie-author", { onModelUpdated: this.onModelUpdated, bundleHost: this.host, defaultComplexRubricModel: this.defaultComplexRubricModel, config: sanctioned, configSettings: configSettings, addPreview: this.addPreview, imageSupport: is, uploadSoundSupport: us, ref: (el) => (this.pieConfigEl = el), isInsidePieApiAuthor: true, bundleEndpoints: this.bundleEndpoints, reFetchBundle: this.reFetchBundle }));
        }
        else {
            return h("div", null, "not loaded");
        }
    }
    static get is() { return "pie-api-author"; }
    static get originalStyleUrls() { return {
        "$": ["pie-api-author.css"]
    }; }
    static get styleUrls() { return {
        "$": ["pie-api-author.css"]
    }; }
    static get properties() { return {
        "bundleEndpoints": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "BundleEndpoints",
                "resolved": "BundleEndpoints",
                "references": {
                    "BundleEndpoints": {
                        "location": "import",
                        "path": "../../types"
                    }
                }
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "Provide custom endpoints for the player components. Can be proxy or direct."
            }
        },
        "host": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "HostEnv",
                "resolved": "\"dev\" | \"local\" | \"prod\" | \"qa\" | \"stage\"",
                "references": {
                    "HostEnv": {
                        "location": "import",
                        "path": "../../types"
                    }
                }
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "Choose 'dev' , 'stage' or 'prod' environments for back end."
            },
            "attribute": "host",
            "reflect": false,
            "defaultValue": "\"prod\""
        },
        "reFetchBundle": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "used to automatically re-fetch the bundle (in case we get a 503); defaults to false"
            },
            "attribute": "re-fetch-bundle",
            "reflect": false,
            "defaultValue": "false"
        },
        "token": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The api token for `pie-api` access."
            },
            "attribute": "token",
            "reflect": true
        },
        "addPreview": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Adds a preview view which will render the content in another tab as it may appear to a student or instructor."
            },
            "attribute": "add-preview",
            "reflect": false,
            "defaultValue": "false"
        },
        "contentConfig": {
            "type": "unknown",
            "mutable": true,
            "complexType": {
                "original": "PieContent",
                "resolved": "PieContent",
                "references": {
                    "PieContent": {
                        "location": "import",
                        "path": "@pie-framework/pie-player-components/dist/types/interface"
                    }
                }
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The model for the content.\nIf set, the view will update to render the provided config."
            }
        },
        "recordScreen": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Enable screen recording with logrocket.\nNote: not to be used in production unless for temporary use to track down a specific user problem."
            },
            "attribute": "record-screen",
            "reflect": false,
            "defaultValue": "false"
        },
        "recordScreenIdentity": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "If screen recording enabled for debug purposes, use this property to provide a unique identifer for\nthe user whose session is being recorded."
            },
            "attribute": "record-screen-identity",
            "reflect": false
        },
        "exceptions": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Enable exception tracking."
            },
            "attribute": "exceptions",
            "reflect": false,
            "defaultValue": "false"
        },
        "configSettings": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "{ [packageName: string]: Object }",
                "resolved": "{ [packageName: string]: Object; }",
                "references": {
                    "Object": {
                        "location": "global"
                    }
                }
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "To customize the standard behaviour provided by interaction configuration views you can\nprovide settings key-ed by the package name.  e.g.\n\n`{ '@pie-element/inline-choice': { promptLabel: 'Item Stem' } }`\n\nThe settings that are configurable for each authoring view are documented in\nthe `@package-name/docs` folder for each package."
            }
        },
        "defaultComplexRubricModel": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "Object",
                "resolved": "Object",
                "references": {
                    "Object": {
                        "location": "global"
                    }
                }
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "Optional. Set a default model for complex-rubric item (works with pie-player-components versions above v1.2.0)."
            }
        },
        "contentId": {
            "type": "string",
            "mutable": true,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "The unique versioned identifier `id@semver-version` for the Pie Content Item stored in the PIE Service CMS."
            },
            "attribute": "content-id",
            "reflect": true
        },
        "collectionId": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "Optional. The unique identifier for the Collection in which the content item will be stored in the PIE Service CMS."
            },
            "attribute": "collection-id",
            "reflect": true
        },
        "overrides": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string | { [key: string]: string }",
                "resolved": "string | { [key: string]: string; }",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Override element versions.\nRequires the 'overrides' scope in the token in pie-api\nmatches the key against the element names and if matching uses the overridden version\nif a string it's parsed as a json string"
            },
            "attribute": "overrides",
            "reflect": false
        },
        "lockVersions": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "If true, the `@version` tag for elements in the configuration will be ignored and a static version for that\ninteraction type determined by the current release of this pie-api-author component will be used."
            },
            "attribute": "lock-versions",
            "reflect": false,
            "defaultValue": "false"
        }
    }; }
    static get contextProps() { return [{
            "name": "doc",
            "context": "document"
        }]; }
    static get states() { return {
        "versions": {},
        "apiConfig": {},
        "lockInProgress": {}
    }; }
    static get events() { return [{
            "method": "contentLoaded",
            "name": "contentLoaded",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "Content has been loaded from server"
            },
            "complexType": {
                "original": "any",
                "resolved": "any",
                "references": {}
            }
        }, {
            "method": "contentIdValidated",
            "name": "contentIdValidated",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "When the `contentId` property is set, it will be parsed to ensure it is a valid identifier and to determine\nif the version is released or pre-release.\n\nthe `event.detail` value will be `{valid: true | false, status: 'released' | 'pre-release'}`"
            },
            "complexType": {
                "original": "any",
                "resolved": "any",
                "references": {}
            }
        }, {
            "method": "recordingStarted",
            "name": "recordingStarted",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "If `recordScreen` property is set to true this event will be\nemitted with the url for recorded screen session in `detail` property."
            },
            "complexType": {
                "original": "any",
                "resolved": "any",
                "references": {}
            }
        }]; }
    static get methods() { return {
        "addRubricToConfig": {
            "complexType": {
                "signature": "(config: ItemConfig, rubricModel?: any) => Promise<ItemConfig>",
                "parameters": [{
                        "tags": [{
                                "text": "config the item config to mutate",
                                "name": "param"
                            }],
                        "text": "the item config to mutate"
                    }, {
                        "tags": [{
                                "text": "rubricModel optional model to pass to `@pie-element/rubric`",
                                "name": "param"
                            }],
                        "text": "optional model to pass to `@pie-element/rubric`"
                    }],
                "references": {
                    "Promise": {
                        "location": "global"
                    },
                    "ItemConfig": {
                        "location": "import",
                        "path": "@pie-framework/pie-player-components/dist/types/interface"
                    },
                    "PieModel": {
                        "location": "import",
                        "path": "@pie-framework/pie-player-components/dist/types/interface"
                    }
                },
                "return": "Promise<ItemConfig>"
            },
            "docs": {
                "text": "Utility method to add a `@pie-element/rubric` section to an item config when creating an item should be used before setting the config.",
                "tags": [{
                        "name": "deprecated",
                        "text": "this method is for temporary use, will be removed at next major release"
                    }, {
                        "name": "param",
                        "text": "config the item config to mutate"
                    }, {
                        "name": "param",
                        "text": "rubricModel optional model to pass to `@pie-element/rubric`"
                    }]
            }
        },
        "load": {
            "complexType": {
                "signature": "(contentId?: any) => Promise<void>",
                "parameters": [{
                        "tags": [{
                                "text": "contentId the unique versioned identifer `id@version` for the content item to load. If this param\nis not provided and `content-id` attribute has a value, that value will be used to load the content item.",
                                "name": "param"
                            }],
                        "text": "the unique versioned identifer `id@version` for the content item to load. If this param\nis not provided and `content-id` attribute has a value, that value will be used to load the content item."
                    }],
                "references": {
                    "Promise": {
                        "location": "global"
                    }
                },
                "return": "Promise<void>"
            },
            "docs": {
                "text": "Loads a content item from `pie-api` host into the view.",
                "tags": [{
                        "name": "param",
                        "text": "contentId the unique versioned identifer `id@version` for the content item to load. If this param\nis not provided and `content-id` attribute has a value, that value will be used to load the content item."
                    }]
            }
        },
        "create": {
            "complexType": {
                "signature": "(preReleaseType?: string) => Promise<string>",
                "parameters": [{
                        "tags": [{
                                "text": "preReleaseType - see preRelaseType for the `save` method",
                                "name": "param"
                            }],
                        "text": "- see preRelaseType for the `save` method"
                    }],
                "references": {
                    "Promise": {
                        "location": "global"
                    }
                },
                "return": "Promise<string>"
            },
            "docs": {
                "text": "Creates a new content item from the currently loaded contents.",
                "tags": [{
                        "name": "param",
                        "text": "preReleaseType - see preRelaseType for the `save` method"
                    }]
            }
        },
        "save": {
            "complexType": {
                "signature": "(preReleaseType?: PreReleaseType) => Promise<string>",
                "parameters": [{
                        "tags": [{
                                "text": "preReleaseType - the type of update to perform",
                                "name": "param"
                            }],
                        "text": "- the type of update to perform"
                    }],
                "references": {
                    "Promise": {
                        "location": "global"
                    },
                    "PreReleaseType": {
                        "location": "import",
                        "path": "../../interface"
                    }
                },
                "return": "Promise<string>"
            },
            "docs": {
                "text": "Saves an update to an existing pre-released content item.\nThe content item version must be in pre-release state.\n\nthe `preReleaseType` param determines what kind of version update, if any, will be performed.\n\n`null` - (default) writes over existing pre-release content, , e.g.`1.0.0-draft.0 -> 1.0.0-draft.0`\\n\n`prerelease` - writes in increment to the prerelease version, e.g. `1.0.0-draft.0 -> 1.0.0-draft.1`\\n\n`premajor` - creates a new major release from version, e.g. `1.0.0-draft.1 -> 2.0.0-draft.0`\\n\n`preminor` - creates a new minor release from version, e.g. `1.0.0-draft.1 -> 1.1.0-draft.0`\\n\n`prepatch` - creates a new patch release from version, e.g. `1.0.0-draft.1 -> 1.0.1-draft.0`\\m",
                "tags": [{
                        "name": "param",
                        "text": "preReleaseType - the type of update to perform"
                    }]
            }
        },
        "version": {
            "complexType": {
                "signature": "() => Promise<string>",
                "parameters": [],
                "references": {
                    "Promise": {
                        "location": "global"
                    }
                },
                "return": "Promise<string>"
            },
            "docs": {
                "text": "",
                "tags": []
            }
        },
        "validate": {
            "complexType": {
                "signature": "() => Promise<any>",
                "parameters": [],
                "references": {
                    "Promise": {
                        "location": "global"
                    }
                },
                "return": "Promise<any>"
            },
            "docs": {
                "text": "this method is for having the possibility to access validateModels from player <br/><br/> It returns the model of the item and errors field which contains the errors returned by validate function in controller. It is an asynchronous method, so it needs to be called with await",
                "tags": []
            }
        },
        "release": {
            "complexType": {
                "signature": "() => Promise<string>",
                "parameters": [],
                "references": {
                    "Promise": {
                        "location": "global"
                    }
                },
                "return": "Promise<string>"
            },
            "docs": {
                "text": "Release a pre-release version of the content currently loaded in the authoring view.\n\nThrows an exception if the version of the current Content Item does not have a pre-release tag e.g.`1.0.1-draft`\n\n  example:\n(`contentId = 'abc@1.0.1-draft`)\n\n  `this.release() => 'abc@1.0.1`'",
                "tags": []
            }
        },
        "getVersions": {
            "complexType": {
                "signature": "() => Promise<string[]>",
                "parameters": [],
                "references": {
                    "Promise": {
                        "location": "global"
                    }
                },
                "return": "Promise<string[]>"
            },
            "docs": {
                "text": "TODO\nGets a list of all versions that exist for this Content Item.",
                "tags": []
            }
        }
    }; }
    static get elementRef() { return "el"; }
    static get watchers() { return [{
            "propName": "host",
            "methodName": "watchHostEnvironment"
        }, {
            "propName": "token",
            "methodName": "onTokenChange"
        }, {
            "propName": "contentId",
            "methodName": "watchContentId"
        }, {
            "propName": "recordScreen",
            "methodName": "watchRecordScreen"
        }]; }
}
