import { h } from "@stencil/core";
import { PlayerApiClient } from '../../clients/player';
import { Mode, Role, } from '../../types';
import { PLAYER_HOST_CONFIG, HOST_DEFAULTS } from '../../defaults';
import debounce from 'lodash/debounce';
import { VERSION } from '../../version';
import { loadFromPieApi } from '../../clients/graphql';
import { splitStimulusItemsBundle } from '../../util';
import { NewRelicInstrumentationService } from '../../clients/new-relic';
export class PieApiPlayer {
    constructor() {
        this.initialized = false;
        /**
         * Use the beta controllers for pie-elements
         */
        this.optimize = false;
        /**
         * Enable screen recording with New Relic.
         * Note: not to be used in production unless for temporary use to track down a specific user problem.
         * Defaults to false
         */
        this.recordScreen = false;
        /**
         * Enable exception tracking with New Relic.
         */
        this.exceptions = false;
        /**
         * Enable tracking page actions/events with New Relic.
         * Note: Setting this to true increases data sent to New Relic, which may affect costs.
         */
        this.trackPageActions = false;
        /**
         * In evaluate mode, add a bottom border to visually separate each item in the case of a multi-item
         */
        this.showBottomBorder = false;
        /**
         * The mode to be used when player is initialized
         */
        this.initialMode = Mode.gather;
        /**
         * The role to be used when player is initialized
         */
        this.initialRole = Role.student;
        /**
         * If the item contains a passage, the player will render it by default.
         * Set this property to false to not render passage.
         */
        this.renderStimulus = true;
        /**
         * It will allow to resize passage and item width for items with stimulus
         */
        this.allowedResize = false;
        /**
         * Choose 'dev' , 'stage' or 'prod' environments for back end.
         */
        this.host = 'prod';
        /**
         * used to automatically re-fetch the bundle (in case we get a 503); defaults to false
         */
        this.reFetchBundle = false;
        /**
         * Customize markup.
         */
        this.customMarkup = '';
        /**
         * If this property is set the item will be rendered in a `preview` state by the player.
         * This means no server-side calls will be made and no reponse-session will be created/stored in pie-api.
         */
        this.preview = false;
    }
    watchHostEnvironment(newVal) {
        if (newVal) {
            this._apiConfig = this._apiConfig || {};
            this._apiConfig.host = HOST_DEFAULTS[newVal].apiUrl;
            this._apiConfig.bundleBaseUrl = HOST_DEFAULTS[newVal].bundleBaseUrl;
        }
    }
    onCustomConfigChange(newValue) {
        console.log('customConfig now', newValue);
    }
    onCustomModelsChange(newValue) {
        console.log('customModels now', newValue);
    }
    onCustomPassageModelsChange(newValue) {
        console.log('customPassageModels now', newValue);
    }
    onCustomMarkupChange(newValue) {
        console.log('customMarkup now', newValue);
    }
    get safeConfig() {
        return Object.assign(Object.assign(Object.assign({}, PLAYER_HOST_CONFIG), this._apiConfig), this.customConfig);
    }
    /**
     * Get the retry configuration from component properties
     */
    getRetryConfig() {
        const config = {};
        if (this.maxRetryAttempts !== undefined)
            config.maxRetryAttempts = this.maxRetryAttempts;
        if (this.requestTimeoutMs !== undefined)
            config.requestTimeoutMs = this.requestTimeoutMs;
        if (this.initialRetryDelayMs !== undefined)
            config.initialRetryDelayMs = this.initialRetryDelayMs;
        if (this.maxRetryDelayMs !== undefined)
            config.maxRetryDelayMs = this.maxRetryDelayMs;
        if (this.trackPageActions !== undefined)
            config.trackPageActions = this.trackPageActions;
        return Object.keys(config).length > 0 ? config : undefined;
    }
    initPlayerClient() {
        if (this.token) {
            this.playerClient = new PlayerApiClient(this.token, this.safeConfig, this.optimize, this.getRetryConfig());
        }
    }
    async watchToken() {
        try {
            this.initPlayerClient();
            await this.load();
            /**
             * Have to call this cos playerInfo is not part of the component state?
             */
            this.el.forceUpdate();
        }
        catch (err) {
            this.trackError('Error during token update', err, {
                errorContext: 'watchToken',
                hasPlayerClient: !!this.playerClient
            });
            this.playerError.emit({
                message: 'Error during token update',
                details: err.message || err,
            });
        }
    }
    async watchItemId(newValue, oldValue) {
        if (newValue !== oldValue) {
            this.updateItemOrSession();
        }
    }
    async watchRetryConfig() {
        // Reinitialize the player client with the new retry configuration
        if (this.token) {
            this.initPlayerClient();
        }
    }
    async watchEnv(newEnv) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r;
        const updatedEnv = Object.assign(Object.assign({}, (this.player && this.player.env
            ? this.player.env
            : {})), newEnv);
        if (this.playerInfo && this.player) {
            try {
                if (!this.preview) {
                    const allModels = await this.playerClient.model(this.player.session, updatedEnv, this.getOverrides());
                    const player = this.player;
                    const models = ((_b = (_a = player) === null || _a === void 0 ? void 0 : _a.config) === null || _b === void 0 ? void 0 : _b.models) || ((_e = (_d = (_c = player) === null || _c === void 0 ? void 0 : _c.config) === null || _d === void 0 ? void 0 : _d.pie) === null || _e === void 0 ? void 0 : _e.models) || [];
                    const passageModels = ((_h = (_g = (_f = player) === null || _f === void 0 ? void 0 : _f.config) === null || _g === void 0 ? void 0 : _g.passage) === null || _h === void 0 ? void 0 : _h.models) || [];
                    // search model based on id and element
                    const findModel = (models, model) => { var _a; return (_a = models) === null || _a === void 0 ? void 0 : _a.find((m) => m.id === model.id && m.element === model.element); };
                    // filter models from allModels
                    const updatedModels = allModels.filter((model) => findModel(models, model));
                    // filter passage models from allModels
                    const updatedPassageModels = allModels.filter((model) => findModel(passageModels, model));
                    const config = this.getConfigParsed(player.config, updatedModels, updatedPassageModels);
                    player.config = config;
                }
                this.player.env = updatedEnv;
                return { error: null };
            }
            catch (e) {
                this.trackError('Environment update error', e, {
                    errorContext: 'watchEnv',
                    originalEnv: JSON.stringify(this.env || {}),
                    newEnv: JSON.stringify(newEnv || {}),
                    updatedEnv: JSON.stringify(updatedEnv || {}),
                    hasPlayerInfo: !!this.playerInfo,
                    hasPlayerConfig: !!((_j = this.player) === null || _j === void 0 ? void 0 : _j.config),
                    modelCount: ((_m = (_l = (_k = this.player) === null || _k === void 0 ? void 0 : _k.config) === null || _l === void 0 ? void 0 : _l.models) === null || _m === void 0 ? void 0 : _m.length) || ((_r = (_q = (_p = (_o = this.player) === null || _o === void 0 ? void 0 : _o.config) === null || _p === void 0 ? void 0 : _p.pie) === null || _q === void 0 ? void 0 : _q.models) === null || _r === void 0 ? void 0 : _r.length) || 0
                });
                this.playerError.emit({
                    message: 'Error updating environment',
                    details: e.message || e,
                });
                return { error: e.message };
            }
        }
    }
    watchRecordScreen(newVal, oldVal) {
        if (newVal && newVal !== oldVal) {
            if (this.newRelic && this.recordScreenIdentity) {
                // Set user identity for New Relic session
                this.newRelic.setUserId(this.recordScreenIdentity);
                // Record session start
                if (this.trackPageActions) {
                    this.newRelic.addPageAction('session_recording_started', {
                        appVersion: VERSION,
                        itemId: this.itemId,
                        sessionId: this.sessionId
                    });
                }
                this.recordingStarted.emit('Session recording started with New Relic');
            }
        }
    }
    /**
     * Return the evaluated score for the current response.
     * The api token must provide permission to authorize the use of this method.
     *
     * Note: Score does not (yet) work when in preview mode - will resolve to null if `preview` is `true`
     *
     * @param disablePartialScoring - if true all interactions will be scored as either correct or incorrect
     */
    async score(disablePartialScoring) {
        if (disablePartialScoring === true || disablePartialScoring === false) {
            console.warn('disablePartialScoring is no longer used - set env instead: {partialScoring: true|false, ...}');
        }
        if (this.preview) {
            return Promise.resolve(null);
        }
        else {
            return this.playerClient.score(this.playerInfo.session, this.env, this.getOverrides());
        }
    }
    async saveSession() {
        var _a, _b, _c, _d;
        if (this.player && this.player.session && !this.preview) {
            try {
                await this.playerClient.saveSession(this.player.session, this.env || {});
                // Track successful session save in New Relic
                if (this.newRelic && this.trackPageActions) {
                    this.newRelic.addPageAction('session_saved', {
                        sessionId: this.sessionId
                    });
                }
                this.sessionSaved.emit(this.sessionId);
            }
            catch (error) {
                console.warn('could not save session');
                this.saveSessionError.emit(error);
                this.trackError('Error saving session', error, {
                    errorContext: 'session-changed event',
                    envMode: ((_a = this.env) === null || _a === void 0 ? void 0 : _a.mode) || this.initialMode,
                    envRole: ((_b = this.env) === null || _b === void 0 ? void 0 : _b.role) || this.initialRole,
                    sessionData: JSON.stringify(((_c = this.player.session) === null || _c === void 0 ? void 0 : _c.data) || []),
                    hasSessionId: !!this.sessionId,
                    sessionPath: ((_d = this.player.session) === null || _d === void 0 ? void 0 : _d.id) ? 'session.id exists' : 'no session.id'
                });
                this.playerError.emit({
                    message: 'Error saving session',
                    details: error.message || error,
                });
            }
        }
    }
    /**
     * Listens for session changed events and saves updated data for session
     */
    handleSessionChange() {
        this.debouncedSaveSession();
    }
    handlePlayerError(ev) {
        this.playerInfo = null;
        this.errorMessage = ev.detail;
        // Collect as much context as possible for player errors
        const errorContext = {
            errorContext: 'player-error event',
            errorPayload: typeof ev.detail === 'object' ? JSON.stringify(ev.detail) : String(ev.detail),
            playerState: this.player ? 'initialized' : 'not initialized',
            playerConfig: this.player && this.player.config ? 'has config' : 'no config',
            currentRoute: typeof window !== 'undefined' ? window.location.pathname : 'unknown'
        };
        this.trackError('Player error', ev.detail || new Error('Unknown player error'), errorContext);
    }
    /**
     * Helper method to track errors with New Relic
     * This is a utility method that centralizes our error reporting logic and ensures
     * we capture comprehensive error information including stack traces.
     */
    trackError(message, error, customAttributes = {}) {
        var _a, _b, _c;
        // First check if newRelic exists to avoid null reference
        if (!this.newRelic) {
            console.error(`[pie-api-player] ${message}:`, error);
            return;
        }
        const shouldReportToNewRelic = this.newRelic && (_a = this.exceptions, (_a !== null && _a !== void 0 ? _a : false));
        if (shouldReportToNewRelic) {
            // Ensure we have an actual Error object with stack trace
            const errorObj = error instanceof Error ? error : new Error(typeof error === 'string' ? error : JSON.stringify(error));
            // Extract stack trace if available
            const stackTrace = errorObj.stack || '';
            // Add default attributes with comprehensive context
            const attributes = Object.assign(Object.assign({}, customAttributes), { itemId: this.itemId, sessionId: this.sessionId, message, appVersion: VERSION, errorMessage: errorObj.message || String(error), stackTrace, componentState: {
                    mode: ((_b = this.env) === null || _b === void 0 ? void 0 : _b.mode) || this.initialMode,
                    role: ((_c = this.env) === null || _c === void 0 ? void 0 : _c.role) || this.initialRole,
                    preview: this.preview
                }, time: new Date().toISOString() });
            // Use New Relic's noticeError API to report the error
            this.newRelic.noticeError(errorObj, attributes);
        }
        // Always log to console for debugging
        console.error(`[pie-api-player] ${message}:`, error, customAttributes);
    }
    async updateItemOrSession() {
        try {
            this.playerInfo = null;
            if (!this.preview) {
                await this.load();
            }
            this.el.forceUpdate();
        }
        catch (err) {
            this.trackError('Error updating item or session', err, {
                errorContext: 'updateItemOrSession',
                itemId: this.itemId,
                sessionId: this.sessionId,
                preview: this.preview,
                updateTriggeredBy: 'property change'
            });
            this.playerError.emit({
                message: 'Error updating item or session',
                details: err.message || err,
            });
        }
    }
    readyToLoad() {
        return (this.itemId || this.sessionId) && this.playerClient && this.token;
    }
    getOverrides() {
        let overrides = undefined;
        try {
            if (typeof this.overrides === 'string') {
                overrides = JSON.parse(this.overrides);
            }
            else {
                if (this.overrides) {
                    overrides = this.overrides;
                }
            }
        }
        catch (e) {
        }
        return overrides;
    }
    async load() {
        var _a, _b;
        try {
            if (this.preview) {
                this.env
                    ? (this.env.role = Role.instructor)
                    : (this.env = { role: Role.instructor, mode: Mode.view });
                const buildPlayerConfig = (response) => {
                    var _a;
                    // TODO pie-api-aws 10. test this once passages are tested
                    const passage = (_a = response) === null || _a === void 0 ? void 0 : _a.passage;
                    const out = {
                        session: { id: '', data: [] },
                    };
                    if (passage) {
                        return Object.assign(Object.assign({}, out), { item: { pie: response.config, passage: passage.config } });
                    }
                    else {
                        return Object.assign(Object.assign({}, out), { item: response.config });
                    }
                };
                if (this.itemId && this.token) {
                    const contentData = await loadFromPieApi(this.token, this.itemId, this.safeConfig.host, this.getOverrides());
                    const info = buildPlayerConfig(contentData);
                    console.log(info);
                    console.log(this);
                    this.modelReload.emit(info);
                    this.playerInfo = info;
                }
                else {
                    this.errorMessage = 'no item id passed to player';
                }
            }
            else {
                if (!this.playerInfo && this.readyToLoad()) {
                    // Track load start in New Relic
                    if (this.newRelic && this.trackPageActions) {
                        this.newRelic.addPageAction('item_load_started', {
                            itemId: this.itemId,
                            sessionId: this.sessionId
                        });
                    }
                    this.playerInfo = await this.playerClient.load(this.itemId, this.sessionId, Object.assign(Object.assign({}, this.env), { mode: this.env && this.env.mode ? this.env.mode : this.initialMode, role: this.env && this.env.role ? this.env.role : this.initialRole }), this.assignmentId, this.getOverrides());
                    // Track successful load in New Relic
                    if (this.newRelic && this.trackPageActions) {
                        this.newRelic.addPageAction('item_loaded', {
                            itemId: this.itemId,
                            sessionId: (_a = this.playerInfo.session) === null || _a === void 0 ? void 0 : _a.id
                        });
                    }
                    let apiInfo = this.playerInfo.item;
                    // is passage model
                    if (apiInfo && apiInfo.pie) {
                        if (this.playerInfo.js && this.playerInfo.js.view) {
                            // deprecated, pie-api should return `bundle.url` prop
                            // we need to create 2 different bundles because for stimulus items we render two pie-player instances PD-4664
                            const [bundle1, bundle2] = splitStimulusItemsBundle(this.playerInfo.js.view);
                            apiInfo.pie.bundle = { url: bundle1 };
                            apiInfo.passage.bundle = { url: bundle2 };
                        }
                    }
                    else {
                        if (this.playerInfo.js && this.playerInfo.js.view) {
                            apiInfo.bundle = { url: this.playerInfo.js.view };
                        }
                    }
                    this.playerInfo.item = apiInfo;
                    if (!this.sessionId &&
                        this.playerInfo.session &&
                        this.playerInfo.session.id) {
                        this.sessionCreated.emit(this.playerInfo.session);
                    }
                }
            }
        }
        catch (err) {
            this.errorMessage = 'error loading item';
            console.error(err);
            this.trackError('Error loading item', err, {
                errorContext: 'load method',
                itemId: this.itemId,
                sessionId: this.sessionId,
                isPreview: this.preview,
                hasToken: !!this.token,
                hasApiConfig: !!this._apiConfig,
                apiHost: (_b = this.safeConfig) === null || _b === void 0 ? void 0 : _b.host,
                loadState: this.playerInfo ? 'has playerInfo' : 'no playerInfo',
                readyToLoad: this.readyToLoad(),
                initialMode: this.initialMode,
                initialRole: this.initialRole,
                hasAssignmentId: !!this.assignmentId,
                hasOverrides: !!this.getOverrides()
            });
            this.playerError.emit({
                message: 'Error loading item',
                details: err.message || err,
            });
        }
    }
    async componentWillLoad() {
        this.debouncedSaveSession = debounce(this.saveSession.bind(this), 100);
        this.newRelic = new NewRelicInstrumentationService(this.doc);
        this.newRelic.setApplicationVersion(`pie-api-components@${VERSION}`);
        this.newRelic.setCustomAttribute('componentType', 'pie-api-player');
        if (this.newRelic && this.trackPageActions) {
            this.newRelic.addPageAction('component_loaded', {
                itemId: this.itemId,
                sessionId: this.sessionId
            });
        }
        if (this.recordScreen) {
            this.watchRecordScreen(this.recordScreen, false);
        }
        this.watchHostEnvironment(this.host);
        this.initPlayerClient();
        if (this.readyToLoad()) {
            await this.load();
        }
    }
    disconnectedCallback() {
        // Record component unload if tracking is enabled
        if (this.newRelic && this.trackPageActions) {
            this.newRelic.addPageAction('component_unloaded', {
                itemId: this.itemId,
                sessionId: this.sessionId
            });
        }
    }
    overrideModels(models, customModels) {
        var _a;
        if (!((_a = customModels) === null || _a === void 0 ? void 0 : _a.length)) {
            return models || [];
        }
        return (models || []).map((model) => (Object.assign(Object.assign({}, model), (customModels.find((cm) => cm.id === model.id) || {}))));
    }
    getConfigParsed(item, models, passageModels) {
        var _a, _b;
        // Check if 'item' has the 'pie' structure
        if (item && item.pie) {
            // Structure with 'pie'
            return Object.assign(Object.assign(Object.assign({}, item), { pie: Object.assign(Object.assign({}, item.pie), { models: this.overrideModels(models || item.pie.models, this.customModels), markup: this.customMarkup || item.pie.markup }) }), (item.passage
                ? {
                    passage: Object.assign(Object.assign({}, item.passage), { models: this.overrideModels(passageModels || item.passage.models, this.customPassageModels) }),
                }
                : {}));
        }
        // Standard structure
        return Object.assign(Object.assign({}, item), { models: this.overrideModels(models || ((_a = item) === null || _a === void 0 ? void 0 : _a.models), this.customModels), markup: this.customMarkup || ((_b = item) === null || _b === void 0 ? void 0 : _b.markup) });
    }
    render() {
        const { item } = this.playerInfo || {};
        const configParsed = this.getConfigParsed(item);
        if (this.playerInfo && this.playerInfo.item) {
            return (h("pie-player", { class: `pie-api-player ${this.customClassname || ''}`.trim(), hosted: !this.preview, bundleHost: this.host, addCorrectResponse: this.preview, config: configParsed, session: this.playerInfo.session, renderStimulus: this.renderStimulus, allowedResize: this.allowedResize, showBottomBorder: this.showBottomBorder, env: {
                    mode: this.env && this.env.mode ? this.env.mode : this.initialMode,
                    role: this.env && this.env.role ? this.env.role : this.initialRole,
                }, ref: (el) => (this.player = el), bundleEndpoints: this.bundleEndpoints, reFetchBundle: this.reFetchBundle, externalStyleUrls: this.externalStyleUrls, customClassname: this.customClassname }, ' '));
        }
        else {
            if (this.errorMessage) {
                return h("error-message", { message: this.errorMessage });
            }
            else {
                return h("div", null, "not yet loaded");
            }
        }
    }
    static get is() { return "pie-api-player"; }
    static get originalStyleUrls() { return {
        "$": ["pie-api-player.css"]
    }; }
    static get styleUrls() { return {
        "$": ["pie-api-player.css"]
    }; }
    static get properties() { return {
        "overrides": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string | { [key: string]: string }",
                "resolved": "string | { [key: string]: string; }",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Override element versions.\nRequires the 'overrides' scope in the token in pie-api\nmatches the key against the element names and if matching uses the overridden version\nif a string it's parsed as a json string"
            },
            "attribute": "overrides",
            "reflect": false
        },
        "optimize": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Use the beta controllers for pie-elements"
            },
            "attribute": "optimize",
            "reflect": false,
            "defaultValue": "false"
        },
        "recordScreen": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Enable screen recording with New Relic.\nNote: not to be used in production unless for temporary use to track down a specific user problem.\nDefaults to false"
            },
            "attribute": "record-screen",
            "reflect": false,
            "defaultValue": "false"
        },
        "recordScreenIdentity": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "If screen recording enabled for debug purposes, use this property to provide a unique identifer for\nthe user whose session is being recorded."
            },
            "attribute": "record-screen-identity",
            "reflect": false
        },
        "exceptions": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Enable exception tracking with New Relic."
            },
            "attribute": "exceptions",
            "reflect": false,
            "defaultValue": "false"
        },
        "trackPageActions": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Enable tracking page actions/events with New Relic.\nNote: Setting this to true increases data sent to New Relic, which may affect costs."
            },
            "attribute": "track-page-actions",
            "reflect": false,
            "defaultValue": "false"
        },
        "showBottomBorder": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "In evaluate mode, add a bottom border to visually separate each item in the case of a multi-item"
            },
            "attribute": "show-bottom-border",
            "reflect": false,
            "defaultValue": "false"
        },
        "initialMode": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The mode to be used when player is initialized"
            },
            "attribute": "initial-mode",
            "reflect": false,
            "defaultValue": "Mode.gather"
        },
        "initialRole": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The role to be used when player is initialized"
            },
            "attribute": "initial-role",
            "reflect": false,
            "defaultValue": "Role.student"
        },
        "token": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "A JWT token to authorize requests on behalf of the player.\nThis token for accessing pie-api can be generated with a valid Api Client Id and Secret."
            },
            "attribute": "token",
            "reflect": false
        },
        "assignmentId": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The identifier for the related assignment.\nThis is only used when loading by itemId."
            },
            "attribute": "assignment-id",
            "reflect": false
        },
        "itemId": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The identifier for the  content to load.\nThe player will immediately load the content when this property is set."
            },
            "attribute": "item-id",
            "reflect": false
        },
        "sessionId": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "If provided, will load the assessment session identified by the id.\nAn assessment session is the record of a student's interaction with the item.\nThe player will immediately load the assessment session when this property is set."
            },
            "attribute": "session-id",
            "reflect": false
        },
        "env": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "Env",
                "resolved": "{ mode: String | Mode; role: String | Role; partialScoring?: boolean; }",
                "references": {
                    "Env": {
                        "location": "import",
                        "path": "../../types"
                    }
                }
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "The current environment context in which to render the player. See notes."
            }
        },
        "renderStimulus": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "If the item contains a passage, the player will render it by default.\nSet this property to false to not render passage."
            },
            "attribute": "render-stimulus",
            "reflect": false,
            "defaultValue": "true"
        },
        "allowedResize": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "It will allow to resize passage and item width for items with stimulus"
            },
            "attribute": "allowed-resize",
            "reflect": false,
            "defaultValue": "false"
        },
        "bundleEndpoints": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "BundleEndpoints",
                "resolved": "BundleEndpoints",
                "references": {
                    "BundleEndpoints": {
                        "location": "import",
                        "path": "../../types"
                    }
                }
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "Provide custom endpoints for the player components. Can be proxy or direct."
            }
        },
        "host": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "HostEnv",
                "resolved": "\"dev\" | \"local\" | \"prod\" | \"qa\" | \"stage\"",
                "references": {
                    "HostEnv": {
                        "location": "import",
                        "path": "../../types"
                    }
                }
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "Choose 'dev' , 'stage' or 'prod' environments for back end."
            },
            "attribute": "host",
            "reflect": false,
            "defaultValue": "'prod'"
        },
        "reFetchBundle": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "used to automatically re-fetch the bundle (in case we get a 503); defaults to false"
            },
            "attribute": "re-fetch-bundle",
            "reflect": false,
            "defaultValue": "false"
        },
        "maxRetryAttempts": {
            "type": "number",
            "mutable": false,
            "complexType": {
                "original": "number",
                "resolved": "number",
                "references": {}
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "Maximum number of retry attempts for failed requests.\nDoes not include the initial request attempt.\nDefault: 2"
            },
            "attribute": "max-retry-attempts",
            "reflect": false
        },
        "requestTimeoutMs": {
            "type": "number",
            "mutable": false,
            "complexType": {
                "original": "number",
                "resolved": "number",
                "references": {}
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "Maximum time in milliseconds to wait for a response before aborting the request.\nUsed to prevent requests from hanging indefinitely in poor network conditions.\nDefault: 10000 (10 seconds)"
            },
            "attribute": "request-timeout-ms",
            "reflect": false
        },
        "initialRetryDelayMs": {
            "type": "number",
            "mutable": false,
            "complexType": {
                "original": "number",
                "resolved": "number",
                "references": {}
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "Initial delay in milliseconds before the first retry attempt.\nThis value is used as the base for exponential backoff calculation.\nDefault: 500 (0.5 seconds)"
            },
            "attribute": "initial-retry-delay-ms",
            "reflect": false
        },
        "maxRetryDelayMs": {
            "type": "number",
            "mutable": false,
            "complexType": {
                "original": "number",
                "resolved": "number",
                "references": {}
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "Maximum delay in milliseconds between retry attempts.\nCaps the exponential backoff to prevent excessively long waits.\nDefault: 2000 (2 seconds)"
            },
            "attribute": "max-retry-delay-ms",
            "reflect": false
        },
        "externalStyleUrls": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Support styling with external stylesheet urls (comma separated urls)\nTarget the pie-player component using the class provided in the class property\nParse the css file and add the class to the stylesheet to scope the styles\nThe last stylesheet url takes priority if the selectors have the same specificity."
            },
            "attribute": "external-style-urls",
            "reflect": false
        },
        "customClassname": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": ""
            },
            "attribute": "custom-classname",
            "reflect": false
        },
        "customConfig": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "ApiEndpoints",
                "resolved": "{ host: string; bundleBaseUrl: string; model: string | MethodAndPath; saveSession: string | MethodAndPath; score: string | MethodAndPath; load: string | MethodAndPath; }",
                "references": {
                    "ApiEndpoints": {
                        "location": "import",
                        "path": "../../types"
                    }
                }
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Customize any of the api endpoint fields."
            }
        },
        "customModels": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "any[]",
                "resolved": "any[]",
                "references": {}
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "Customize model."
            }
        },
        "customPassageModels": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "any[]",
                "resolved": "any[]",
                "references": {}
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "Customize passage models."
            }
        },
        "customMarkup": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": true,
            "docs": {
                "tags": [],
                "text": "Customize markup."
            },
            "attribute": "custom-markup",
            "reflect": false,
            "defaultValue": "''"
        },
        "preview": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "If this property is set the item will be rendered in a `preview` state by the player.\nThis means no server-side calls will be made and no reponse-session will be created/stored in pie-api."
            },
            "attribute": "preview",
            "reflect": false,
            "defaultValue": "false"
        }
    }; }
    static get contextProps() { return [{
            "name": "doc",
            "context": "document"
        }]; }
    static get states() { return {
        "_apiConfig": {},
        "errorMessage": {}
    }; }
    static get events() { return [{
            "method": "recordingStarted",
            "name": "recordingStarted",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "If `recordUserSession` property is set to true, detailed user session data will be sent to New Relic\nfor troubleshooting purposes."
            },
            "complexType": {
                "original": "any",
                "resolved": "any",
                "references": {}
            }
        }, {
            "method": "saveSessionError",
            "name": "saveSessionError",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "The player will auto-save response session data when the response is changed by the user.\nIf it encounters an error saving this response to the item session, this event will be emitted.\nThe event detail will contain the exception thrown by the attempted save."
            },
            "complexType": {
                "original": "any",
                "resolved": "any",
                "references": {}
            }
        }, {
            "method": "sessionSaved",
            "name": "sessionSaved",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "pie-api-player listens for `session-changed`events from pie-player and auto-saves the response to pie-api.\nThis event will be emitted when that request completes successfully. If not a `saveSessionError`event will be emitted.\nThe event detail will contain the session id of the session that was saved."
            },
            "complexType": {
                "original": "any",
                "resolved": "any",
                "references": {}
            }
        }, {
            "method": "modelReload",
            "name": "model-reload",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "Emitted when the model was reloaded, content item query was called with the new content version."
            },
            "complexType": {
                "original": "any",
                "resolved": "any",
                "references": {}
            }
        }, {
            "method": "playerError",
            "name": "api-player-error",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "Emitted when the `pie-api-player` encounters an error in the following cases:\n- Loading the item or session.\n- Updating the item, session, or token.\n- Environment configuration issues.\n- Saving session data fails.\n\nEvent detail contains:\n- `message`: A high-level description of the error.\n- `details`: Detailed error information or stack trace.\n\nExample usage:\n```javascript\nplayer.addEventListener(\"api-player-error\", (event) => {\n   console.error(\"Error from API Player:\", event.detail);\n});\n```"
            },
            "complexType": {
                "original": "any",
                "resolved": "any",
                "references": {}
            }
        }, {
            "method": "sessionCreated",
            "name": "session-created",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "Emitted when an item has loaded with a sessionId that was not specified on load.\nThe event detail contains the generated session id and data.\n\n`event.detail === {id: session_id, data:[session_data]}`"
            },
            "complexType": {
                "original": "any",
                "resolved": "any",
                "references": {}
            }
        }]; }
    static get methods() { return {
        "score": {
            "complexType": {
                "signature": "(disablePartialScoring?: boolean) => Promise<any>",
                "parameters": [{
                        "tags": [{
                                "text": "disablePartialScoring - if true all interactions will be scored as either correct or incorrect",
                                "name": "param"
                            }],
                        "text": "- if true all interactions will be scored as either correct or incorrect"
                    }],
                "references": {
                    "Promise": {
                        "location": "global"
                    }
                },
                "return": "Promise<any>"
            },
            "docs": {
                "text": "Return the evaluated score for the current response.\nThe api token must provide permission to authorize the use of this method.\n\nNote: Score does not (yet) work when in preview mode - will resolve to null if `preview` is `true`",
                "tags": [{
                        "name": "param",
                        "text": "disablePartialScoring - if true all interactions will be scored as either correct or incorrect"
                    }]
            }
        },
        "saveSession": {
            "complexType": {
                "signature": "() => Promise<void>",
                "parameters": [],
                "references": {
                    "Promise": {
                        "location": "global"
                    }
                },
                "return": "Promise<void>"
            },
            "docs": {
                "text": "",
                "tags": []
            }
        }
    }; }
    static get elementRef() { return "el"; }
    static get watchers() { return [{
            "propName": "host",
            "methodName": "watchHostEnvironment"
        }, {
            "propName": "customConfig",
            "methodName": "onCustomConfigChange"
        }, {
            "propName": "customModels",
            "methodName": "onCustomModelsChange"
        }, {
            "propName": "customPassageModels",
            "methodName": "onCustomPassageModelsChange"
        }, {
            "propName": "customMarkup",
            "methodName": "onCustomMarkupChange"
        }, {
            "propName": "token",
            "methodName": "watchToken"
        }, {
            "propName": "sessionId",
            "methodName": "watchItemId"
        }, {
            "propName": "itemId",
            "methodName": "watchItemId"
        }, {
            "propName": "maxRetryAttempts",
            "methodName": "watchRetryConfig"
        }, {
            "propName": "requestTimeoutMs",
            "methodName": "watchRetryConfig"
        }, {
            "propName": "initialRetryDelayMs",
            "methodName": "watchRetryConfig"
        }, {
            "propName": "maxRetryDelayMs",
            "methodName": "watchRetryConfig"
        }, {
            "propName": "trackPageActions",
            "methodName": "watchRetryConfig"
        }, {
            "propName": "env",
            "methodName": "watchEnv"
        }, {
            "propName": "recordScreen",
            "methodName": "watchRecordScreen"
        }]; }
    static get listeners() { return [{
            "name": "session-changed",
            "method": "handleSessionChange",
            "target": undefined,
            "capture": false,
            "passive": false
        }, {
            "name": "player-error",
            "method": "handlePlayerError",
            "target": undefined,
            "capture": false,
            "passive": false
        }]; }
}
