import { parse } from "semver";
import parseNpm from "parse-package-name";
import cloneDeep from "lodash/cloneDeep";
export const getVersionedId = (versionedId) => {
    if (!versionedId)
        return null;
    const [id, version] = versionedId.split("@");
    const parsed = parse(version);
    if (version && !parsed)
        return null; // badly formatted version
    if (!id)
        return null;
    const parsedVersion = parsed && parsed.version
        ? {
            major: parsed ? parsed.major : null,
            minor: parsed ? parsed.minor : null,
            patch: parsed ? parsed.patch : null,
            prerelease: {
                tag: parsed ? parsed.prerelease[0] : null,
                version: parsed ? parsed.prerelease[1] : null,
            },
        }
        : null;
    return {
        id,
        version: parsedVersion,
    };
};
export const versionedIdtoString = (vId) => {
    try {
        if (!vId && vId.version)
            return null;
        const prTag = vId.version.prerelease && vId.version.prerelease.tag;
        const prVersion = vId.version.prerelease &&
            typeof vId.version.prerelease.version === "number"
            ? vId.version.prerelease.version.toString()
            : "";
        // prerelease must at least have tag
        const prString = prTag
            ? `-${prTag}${prVersion ? "." + prVersion : ""}`
            : "";
        return `${vId.id}@${vId.version.major}.${vId.version.minor}.${vId.version.patch}${prString}`;
    }
    catch (_a) {
        return null;
    }
};
export const makeGuid = () => {
    function S4() {
        return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1);
    }
    return (S4() +
        S4() +
        "-" +
        S4() +
        "-4" +
        S4().substr(0, 3) +
        "-" +
        S4() +
        "-" +
        S4() +
        S4() +
        S4()).toLowerCase();
};
export const isPreRelease = (id) => {
    return !!(id &&
        id.version &&
        (id.version.major || id.version.major === 0) &&
        (id.version.minor || id.version.minor === 0) &&
        (id.version.patch || id.version.patch === 0) &&
        id.version.prerelease &&
        id.version.prerelease.tag);
};
export const idInfo = (id) => {
    const vId = getVersionedId(id);
    return id && vId
        ? isPreRelease(getVersionedId(id))
            ? "pre-release"
            : "released"
        : void 0;
};
export const lockConfigVersions = (config, versions, overrides) => {
    if (!versions) {
        console.warn("Error loading versions - dont change the config");
        return config;
    }
    const out = cloneDeep(config);
    const elements = out && out.elements;
    if (elements) {
        const keys = Object.keys(elements);
        keys.forEach((key) => {
            const v = elements[key];
            const packageName = parseNpm(v).name;
            if (versions[packageName]) {
                const mk = Object.keys(overrides || {}).find((o) => o === packageName || new RegExp(o).test(packageName));
                const v = mk ? overrides[mk] : versions[packageName];
                elements[key] = packageName + "@" + v;
            }
        });
    }
    return out;
};
export const unlockConfigVersions = (config) => {
    const out = cloneDeep(config);
    const elements = out && out.elements;
    if (elements) {
        const keys = Object.keys(elements);
        keys.forEach((key) => {
            const v = elements[key];
            const packageName = parseNpm(v).name;
            elements[key] = packageName + "@latest";
        });
    }
    return out;
};
export const splitStimulusItemsBundle = (bundleUrl) => {
    if (bundleUrl && bundleUrl[0]) {
        const url = bundleUrl[0];
        const baseUrl = url.split('/bundles/')[0]; // Get the base URL
        const bundlePath = url.split('/bundles/')[1]; // Get the bundles part after /bundles/
        // /load endpoint always returns player.js bundle
        const [bundleList,] = bundlePath.split('/player.js'); // Separate bundles and file
        // backend joins bundles by +
        const components = bundleList.split('+'); // Split bundles by '+'
        // Group components for each new bundle
        const group1 = components.filter(bundle => !bundle.includes('@pie-element/passage'));
        const group2 = components.filter(bundle => bundle.includes('@pie-element/passage'));
        // Create new URLs
        const url1 = `${baseUrl}/bundles/${group1.join('+')}/player.js`;
        const url2 = `${baseUrl}/bundles/${group2.join('+')}/player.js`;
        return [[url1], [url2]];
    }
    return [[], []];
};
