import { r as registerInstance, c as createEvent, d as getContext, h, e as getElement } from './core-43f43fa0.js';
import { c as createCommonjsModule, u as unwrapExports } from './_commonjsHelpers-91036208.js';
import './index-801c8326.js';
import './isSymbol-f32647bc.js';
import { n as normalizeContentElements } from './utils-77e24cb0.js';
import { b as addRubric } from './get-b9d5808f.js';
import { V as VERSION, P as PieLoader, D as DEFAULT_ENDPOINTS, B as BundleType, m as mathRenderingAccessible$1 } from './version-a0ee00c5.js';

var lib = createCommonjsModule(function (module, exports) {
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class ModelSetEvent extends CustomEvent {
    constructor(component, complete, hasModel) {
        super(ModelSetEvent.TYPE, { bubbles: true, composed: true, detail: { complete, component, hasModel } });
        this.component = component;
        this.complete = complete;
    }
}
ModelSetEvent.TYPE = 'model-set';
exports.ModelSetEvent = ModelSetEvent;
class SessionChangedEvent extends CustomEvent {
    constructor(component, complete) {
        super(SessionChangedEvent.TYPE, { bubbles: true, composed: true, detail: { complete, component } });
        this.component = component;
        this.complete = complete;
    }
}
SessionChangedEvent.TYPE = 'session-changed';
exports.SessionChangedEvent = SessionChangedEvent;

});

const index = unwrapExports(lib);
var lib_1 = lib.ModelSetEvent;
var lib_2 = lib.SessionChangedEvent;

const controllerErrorMessage = "Error processing question configuration, verify the question model?";
const Player = class {
    constructor(hostRef) {
        registerInstance(this, hostRef);
        /**
         * Allows disabling of the default behaviour which is to look up and load the JS bundle that define the Custom Elements
         * used by the item config.
         * This if for advanced use cases when using the pie-player in a container that is managing loading of Custom Elements and Controllers.
         */
        this.disableBundler = false;
        this.elementsLoaded = false;
        /**
         * Simulates a correct response for the item.
         * This property will only have this effect if the `hosted` property is
         * false and player is running client-side-only.
         */
        this.addCorrectResponse = false;
        /**
         * In evaluate mode, add a bottom border to visually separate each item in the case of a multi-item
         */
        this.showBottomBorder = false;
        /**
         * used to automatically re-fetch the bundle (in case we get a 503)
         */
        this.reFetchBundle = false;
        /**
         * The Pie Session
         */
        this.session = { id: "", data: [] };
        /**
         * Describes runtime environment for the player.
         *
         */
        this.env = { mode: "gather", role: "student" };
        /**
         * Indicates if player running in the context of a PIE hosting system.
         * Do not modify the default value for this property if you are not implementing a PIE host.
         * If true, the host is responsible for all model updates.
         */
        this.hosted = false;
        /**
         * If the item contains a stimulus, the player will render it by default.
         * Set this property to false to not render stimulus.
         */
        this.renderStimulus = true;
        this.version = VERSION;
        /**
         * Allow to resize pie-stimulus layout
         * Set this property to false to not render the resizer.
         */
        this.allowedResize = false;
        this.pieLoader = new PieLoader(null, this.loaderConfig);
        this.sessionChanged = createEvent(this, "session-changed", 7);
        this.playerError = createEvent(this, "player-error", 7);
        this.responseCompleted = createEvent(this, "responseCompleted", 7);
        this.loadComplete = createEvent(this, "load-complete", 7);
        this.doc = getContext(this, "document");
    }
    watchAddCorrectResponse(newValue, oldValue) {
        if (newValue !== oldValue) {
            this.updateModels();
        }
    }
    player() {
        return this.stimulusPlayer ? this.stimulusPlayer : this;
    }
    async watchConfig(newConfig) {
        this.elementsLoaded = false;
        try {
            if (!newConfig) {
                return;
            }
            try {
                if (typeof newConfig == "string") {
                    newConfig = JSON.parse(newConfig);
                }
                if (newConfig.pie) {
                    this.stimulusItemModel = newConfig;
                    return; // if stimulus item
                }
                else if (newConfig.elements) {
                    this.pieContentModel = addRubric(newConfig);
                    this.pieContentModel = normalizeContentElements(this.pieContentModel);
                }
                else {
                    this.playerError.emit(`invalid pie data model`);
                    return;
                }
            }
            catch (err) {
                this.playerError.emit(`exception processing content model - ${err.message}`);
                return;
            }
            if (!this.elementsLoaded && !this.disableBundler) {
                let endpoints = DEFAULT_ENDPOINTS.prod;
                if (this.bundleHost &&
                    ["dev", "stage", "prod"].includes(this.bundleHost)) {
                    endpoints = DEFAULT_ENDPOINTS[this.bundleHost];
                }
                // if bundleEndpoints are provided, use them instead of the default
                // used for proxies, etc.
                let forceBundleUrl = false;
                if (this.bundleEndpoints) {
                    endpoints = this.bundleEndpoints;
                    forceBundleUrl = true;
                }
                await this.pieLoader.loadCloudPies({
                    content: this.pieContentModel,
                    doc: this.doc,
                    endpoints: endpoints,
                    bundle: this.hosted ? BundleType.player : BundleType.clientPlayer,
                    useCdn: false,
                    forceBundleUrl,
                    reFetchBundle: this.reFetchBundle
                });
            }
        }
        catch (err) {
            this.playerError.emit(`problem loading item (${err})`);
        }
    }
    /**
     * For previewing changes to an item. Updates the model for one question in the item model.
     * @param update the updated model
     */
    async updateElementModel(update) {
        if (this.pieContentModel && this.pieContentModel.models) {
            const index = this.pieContentModel.models.findIndex(m => m.id === update.id);
            if (index !== -1) {
                this.pieContentModel.models.splice(index, 1, update);
            }
            await this.updateModels();
        }
    }
    async provideScore() {
        // TODO I think we have to check if it's hosted or not to expose this function
        if (!this.pieContentModel || !this.pieContentModel.models) {
            console.error("No pie content model");
            return false;
        }
        return Promise.all((this.pieContentModel.models || []).map(async (model) => {
            let pieEl = this.el.querySelector(`[id='${model.id}']`);
            !pieEl && (pieEl = this.el.querySelector(`[pie-id='${model.id}']`));
            const session = this.findOrAddSession(this.session.data, model.id);
            if (pieEl) {
                const controller = this.pieLoader.getController(pieEl.localName);
                if (controller && controller.outcome) {
                    return Object.assign(Object.assign({}, session), (await controller.outcome(model, session, {
                        mode: "evaluate"
                    })));
                }
            }
        }));
    }
    updateModels(newEnv = this.env) {
        // wrapping a player in stimulus layout
        if (this.stimulusPlayer) {
            this.stimulusPlayer.env = newEnv;
            return;
        }
        performance.mark("pie-model-set-start");
        if (this.pieContentModel &&
            this.pieContentModel.models &&
            typeof this.pieContentModel.models.forEach === "function" &&
            this.pieContentModel.markup &&
            this.elementsLoaded) {
            /**
             * Block session changed events while we set model/session on the elements.
             * TODO: The elements should *not* be firing 'session-changed' when the session is set.
             * They should only fire this if a user has made a change. Can we guarantee that?
             */
            this.el.addEventListener(lib_2.TYPE, this.stopEventFromPropagating);
            this.pieContentModel.models.forEach(async (model, index) => {
                if (model && model.error) {
                    this.playerError.emit(`error loading question data`);
                    throw new Error(model.error);
                }
                const pieEl = this.el.querySelector(`[id='${model.id}']`);
                let session = this.findOrAddSession(this.session.data, model.id);
                if (pieEl) {
                    if (!this.hosted) {
                        try {
                            // use local controllers
                            const controller = this.pieLoader.getController(pieEl.localName);
                            if (controller) {
                                if (this.addCorrectResponse &&
                                    controller.createCorrectResponseSession &&
                                    typeof controller.createCorrectResponseSession === "function") {
                                    session = await controller.createCorrectResponseSession(model, Object.assign(Object.assign({}, newEnv), { role: "instructor" }));
                                }
                                pieEl.model = await controller.model(model, session, newEnv);
                            }
                            else {
                                // no controller provided
                                pieEl.model = model;
                            }
                        }
                        catch (err) {
                            this.playerError.emit(`${controllerErrorMessage}  -  (${err})`);
                        }
                    }
                    else {
                        if (model.error) {
                            this.playerError.emit(`${controllerErrorMessage}  -  '${model.error}'`);
                        }
                        pieEl.model = model;
                        if (this.addCorrectResponse) {
                            this.playerError.emit(`add-correct-response cannot be used in hosted environement`);
                        }
                    }
                    try {
                        pieEl.session = session;
                    }
                    catch (err) {
                        this.playerError.emit(`error setting item session value - ${err.message}`);
                    }
                }
            });
            performance.mark("pie-model-set-end");
            performance.measure("PIE Model Setup Time", "pie-model-set-start", "pie-model-set-end");
            const modelEntry = performance.getEntriesByName("PIE Model Setup Time")[0];
            const modelDuration = modelEntry ? modelEntry.duration : 0;
            console.log("[PIE Model Setup Time]", modelDuration.toFixed(2), "ms");
            setTimeout(() => {
                /** remove the event blocker - see above */
                this.el.removeEventListener(lib_2.TYPE, this.stopEventFromPropagating);
                //TODO: is this needed anymore? maybe it should be using 'model-set' instead of 'session-changed' which is what it was using.
                this.loadComplete.emit();
            }, 150);
        }
    }
    stopEventFromPropagating(e) {
        e.stopPropagation();
    }
    scopeCSS(cssText, customClassname) {
        return cssText.replace(/(^|\})\s*([^{]+)\s*\{/g, (_, close, selector) => {
            const scoped = selector
                .split(",")
                .map((s) => `pie-player.${customClassname} ${s.trim()}`)
                .join(", ");
            return `${close} ${scoped} {`;
        });
    }
    async loadScopedExternalStyle(url, customClassname) {
        const startTime = performance.now();
        console.log(`[style-loader] Fetching CSS: ${url}`);
        const res = await fetch(url);
        const css = await res.text();
        const fetchEnd = performance.now();
        console.log(`[style-loader] Fetch complete in ${(fetchEnd - startTime).toFixed(2)}ms`);
        const scopeStart = performance.now();
        const scopedCSS = this.scopeCSS(css, customClassname);
        const scopeEnd = performance.now();
        console.log(`[style-loader] CSS scoped in ${(scopeEnd - scopeStart).toFixed(2)}ms`);
        const styleTag = document.createElement("style");
        styleTag.setAttribute("data-pie-style", url);
        styleTag.textContent = scopedCSS;
        document.head.appendChild(styleTag);
    }
    async componentWillLoad() {
        if (this.config) {
            this.watchConfig(this.config);
        }
    }
    findOrAddSession(data, id) {
        const s = data.find(d => d.id === id);
        if (s) {
            return s;
        }
        const ss = { id };
        data.push(ss);
        return ss;
    }
    renderMath() {
        mathRenderingAccessible$1.renderMath(this.el);
    }
    addBottomBorder(tags) {
        if (!Array.isArray(tags)) {
            return;
        }
        tags.forEach(tag => {
            const elems = document.querySelectorAll(`${tag}`);
            for (const elem of elems) {
                if (elem && elem instanceof HTMLElement) {
                    elem.classList.add("evaluate-bottom-border");
                }
            }
        });
    }
    async afterRender() {
        if (this.pieContentModel && this.pieContentModel.markup) {
            if (this.elementsLoaded) {
                this.updateModels();
                this.renderMath();
                if (this.showBottomBorder && this.env.mode === "evaluate") {
                    const pieTags = this.pieContentModel.elements &&
                        Object.keys(this.pieContentModel.elements);
                    this.addBottomBorder(pieTags);
                }
            }
            else {
                const elements = Object.keys(this.pieContentModel.elements).map(el => ({
                    name: el,
                    tag: el
                }));
                // Note: hard to verify but it appears that we need to resolve
                // the value first rather than setting the promise directly on
                // this state property - otherwise lifecycle re-render is triggered too early
                const loadedInfo = await this.pieLoader.elementsHaveLoaded(elements);
                if (loadedInfo.val &&
                    !!loadedInfo.elements.find(el => this.pieContentModel.elements[el.name])) {
                    this.elementsLoaded = true;
                    if (performance.getEntriesByName("pie-load-end").length > 0) {
                        performance.mark("pie-elements-load-end");
                        performance.measure("PIE Elements Load Time", "pie-load-end", "pie-elements-load-end");
                        const renderEntry = performance.getEntriesByName("PIE Elements Load Time")[0];
                        const renderDuration = renderEntry ? renderEntry.duration : 0;
                        console.log("[PIE Elements Load + Render Time]", renderDuration.toFixed(2), "ms");
                    }
                }
            }
        }
        if (this.externalStyleUrls &&
            typeof this.externalStyleUrls === "string" &&
            this.customClassname &&
            typeof this.customClassname === "string") {
            const urls = this.externalStyleUrls.split(",").map(url => url.trim());
            urls.forEach(url => {
                if (url &&
                    typeof url === "string" &&
                    !document.querySelector(`style[data-pie-style="${url}"]`)) {
                    this.loadScopedExternalStyle(url, this.customClassname);
                }
            });
        }
    }
    async componentDidLoad() {
        await this.afterRender();
    }
    async componentDidUpdate() {
        await this.afterRender();
    }
    render() {
        if (this.stimulusItemModel) {
            return this.renderStimulus ? (h("pie-stimulus-layout", { allowedResize: this.allowedResize }, h("div", { slot: "stimulus", class: "player-stimulus-container" }, h("pie-player", { id: "stimulusPlayer", config: this.stimulusItemModel.passage, env: this.env, hosted: this.hosted, session: this.session, ref: el => (this.stimulusPlayer = el), bundleHost: this.bundleHost, bundleEndpoints: this.bundleEndpoints, class: this.customClassname })), h("div", { slot: "item", class: "player-item-container" }, h("pie-player", { id: "itemPlayer", addCorrectResponse: this.addCorrectResponse, config: this.stimulusItemModel.pie, env: this.env, hosted: this.hosted, session: this.session, bundleHost: this.bundleHost, bundleEndpoints: this.bundleEndpoints, class: this.customClassname })))) : (h("pie-player", { id: "itemPlayer", addCorrectResponse: this.addCorrectResponse, config: this.stimulusItemModel.pie, env: this.env, hosted: this.hosted, session: this.session, bundleEndpoints: this.bundleEndpoints, class: this.customClassname }));
        }
        else {
            if (this.elementsLoaded) {
                return (h("div", { class: "player-container", innerHTML: this.pieContentModel && this.pieContentModel.markup
                        ? this.pieContentModel.markup
                        : "" }));
            }
            return h("pie-spinner", null);
        }
    }
    get el() { return getElement(this); }
    static get watchers() { return {
        "addCorrectResponse": ["watchAddCorrectResponse"],
        "config": ["watchConfig"],
        "env": ["updateModels"]
    }; }
    static get style() { return ".pie-loading {\n  position: absolute;\n  height: 100%;\n  width: 100%;\n  background-color: #fff;\n  bottom: 0;\n  left: 0;\n  right: 0;\n  top: 0;\n  z-index: 9999;\n  opacity: 0.4;\n}\n\n.pie-loading.fixed {\n  position: fixed;\n}\n\n.pie-loading:before {\n  content: \"\";\n  background-color: rgba(0, 0, 0, 0);\n  border: 5px solid rgba(63, 81, 181, 0.9);\n  opacity: 0.9;\n  border-right: 5px solid rgba(0, 0, 0, 0);\n  border-left: 5px solid rgba(0, 0, 0, 0);\n  border-radius: 50px;\n  -webkit-box-shadow: 0 0 35px rgba(63, 81, 181, 0.9);\n  box-shadow: 0 0 35px rgba(63, 81, 181, 0.9);\n  width: 50px;\n  height: 50px;\n  -moz-animation: spinPulse 1s infinite ease-in-out;\n  -webkit-animation: spinPulse 1s infinite linear;\n  margin: -25px 0 0 -25px;\n  position: absolute;\n  top: 50%;\n  left: 50%;\n}\n\n.pie-loading:after {\n  content: \"\";\n  background-color: rgba(0, 0, 0, 0);\n  border: 5px solid rgba(63, 81, 181, 0.9);\n  opacity: 0.9;\n  border-left: 5px solid rgba(0, 0, 0, 0);\n  border-right: 5px solid rgba(0, 0, 0, 0);\n  border-radius: 50px;\n  -webkit-box-shadow: 0 0 15px rgba(63, 81, 181, 0.9);\n  box-shadow: 0 0 15px rgba(63, 81, 181, 0.9);\n  width: 30px;\n  height: 30px;\n  -moz-animation: spinoffPulse 1s infinite linear;\n  -webkit-animation: spinoffPulse 1s infinite linear;\n  margin: -15px 0 0 -15px;\n  position: absolute;\n  top: 50%;\n  left: 50%;\n}\n\n\@-webkit-keyframes spinPulse {\n  0% {\n    -webkit-transform: rotate(160deg);\n    transform: rotate(160deg);\n    opacity: 0;\n    -webkit-box-shadow: 0 0 1px rgba(63, 81, 181, 0.9);\n    box-shadow: 0 0 1px rgba(63, 81, 181, 0.9);\n  }\n  50% {\n    -webkit-transform: rotate(145deg);\n    transform: rotate(145deg);\n    opacity: 1;\n  }\n  100% {\n    -webkit-transform: rotate(-320deg);\n    transform: rotate(-320deg);\n    opacity: 0;\n  }\n}\n\n\@keyframes spinPulse {\n  0% {\n    -webkit-transform: rotate(160deg);\n    transform: rotate(160deg);\n    opacity: 0;\n    -webkit-box-shadow: 0 0 1px rgba(63, 81, 181, 0.9);\n    box-shadow: 0 0 1px rgba(63, 81, 181, 0.9);\n  }\n  50% {\n    -webkit-transform: rotate(145deg);\n    transform: rotate(145deg);\n    opacity: 1;\n  }\n  100% {\n    -webkit-transform: rotate(-320deg);\n    transform: rotate(-320deg);\n    opacity: 0;\n  }\n}\n\n\@-webkit-keyframes spinoffPulse {\n  0% {\n    -webkit-transform: rotate(0deg);\n    transform: rotate(0deg);\n  }\n  100% {\n    -webkit-transform: rotate(360deg);\n    transform: rotate(360deg);\n  }\n}\n\n\@keyframes spinoffPulse {\n  0% {\n    -webkit-transform: rotate(0deg);\n    transform: rotate(0deg);\n  }\n  100% {\n    -webkit-transform: rotate(360deg);\n    transform: rotate(360deg);\n  }\n}\n\n.pie-api-player img {\n  max-width: inherit !important;\n  max-height: inherit !important;\n}\n\n/* These styles are used in the body content of an item -- they also appear in the item body editor. */\n\n.book-title-k5 {\n  text-decoration: underline;\n}\n\n.book-title {\n  font-style: italic;\n}\n\n.block-quote {\n  font-family: serif;\n  margin-left: 20%;\n  margin-right: 20%;\n  margin-top: 5px;\n  margin-bottom: 5px;\n  width: 60%;\n  border: 1px solid black;\n  padding: 2px;\n  display: block;\n}\n\n.text-block {\n  font-family: serif;\n  margin-left: 20%;\n  margin-right: 20%;\n  margin-top: 5px;\n  margin-bottom: 5px;\n  width: 60%;\n  border: 1px solid black;\n  padding: 2px;\n  display: block;\n}\n\n.short-quote {\n  font-weight: bold;\n}\n\nspan.short-quote:before {\n  content: open-quote;\n}\n\nspan.short-quote:after {\n  content: close-quote;\n}\n\n.word-callout {\n  text-decoration: underline;\n}\n\n.relative-emphasis {\n  text-transform: uppercase;\n}\n\n.content-emphasis {\n  font-weight: bold;\n  color: red;\n}\n\nspan.passage-title:before {\n  content: open-quote;\n}\n\nspan.passage-title:after {\n  content: close-quote;\n}\n\n.proper-name {\n  font-style: italic;\n}\n\n.variable {\n  font-style: italic;\n}\n\n.equation-block {\n  text-align: center;\n  margin-left: 20%;\n  margin-right: 20%;\n  margin-top: 5px;\n  margin-bottom: 5px;\n  width: 60%;\n  padding: 2px;\n  display: block;\n}\n\n.embedded-error {\n  font-style: italic;\n}\n\n/* PASSAGES */\n\ndiv.passage-title,\ndiv.passage-subtitle {\n  font-weight: bold;\n  text-align: center;\n}\n\ndiv.passage-author {\n  text-align: center;\n}\n\n.p-number {\n  float: left;\n  font-size: 0.85em;\n}\n\n.numbered-paragraph {\n  margin-left: 36px;\n}\n\n/* ch5680 - also temporary kds content styles that need to be externalized */\n/* Inspect Styles */\n\n\@media print {\n  .kds-noprint {\n    display: none;\n  }\n\n  .noprint {\n    display: none;\n  }\n}\n\n.kds-absolute {\n  border-left: 1px solid black;\n  border-right: 1px solid black;\n  padding: 0px 0.3em 0px 0.3em;\n}\n\n.abs {\n  border-left: #000000 1px solid;\n  border-right: #000000 1px solid;\n  padding: 0px 0.3em 0px 0.3em;\n}\n\n.kds-border-1 {\n  border: 1px solid black;\n}\n\n.kds-border-2 {\n  border: 2px solid black;\n}\n\n.kds-center {\n  margin: 0px;\n  text-align: center;\n}\n\n.center {\n  text-align: center;\n  margin: 0;\n}\n\n.kds-exponent {\n  font-size: 85%;\n  margin-left: -0.4em;\n  position: relative;\n  top: -1.4em;\n}\n\nsup.frac {\n  position: relative;\n  top: -1.4em;\n  font-size: 70%;\n  margin-left: -0.4em;\n}\n\n.kds-fillin {\n  border: 1px solid black;\n  display: inline-block;\n  margin: 0em 0.2em;\n  padding-left: 1em;\n}\n\n.fillin {\n  margin: 0em 0.2em;\n  padding-left: 1em;\n  border: #000000 1px solid;\n  display: inline-block;\n  *display: inline;\n}\n\n.kds-font-larger {\n  font-size: larger;\n}\n\n.kds-font-smaller {\n  font-size: smaller;\n}\n\ntable.kds-fraction {\n  border: 0px;\n  display: inline-block;\n  font-size: 85%;\n  margin: 0px;\n  padding: 0px;\n  vertical-align: middle;\n}\n\ntable.kds-fraction > tbody > tr > td.kds-numerator {\n  border-bottom: 1px solid black;\n  border-left: 0px;\n  border-right: 0px;\n  border-top: 0px;\n  text-align: center;\n}\n\ntable.kds-fraction > tbody > tr > td.kds-denominator {\n  border: 0px;\n  text-align: center;\n}\n\ntable.frac {\n  display: inline-block;\n  *display: inline;\n  vertical-align: middle;\n  font-size: 85%;\n  border: 0px;\n  margin: 0px;\n  padding: 0px;\n}\n\n.frac .nu {\n  border-left: 0px;\n  border-top: 0px;\n  border-right: 0px;\n  border-bottom: #000000 1px solid;\n  text-align: center;\n}\n\n.frac .de {\n  border: 0px;\n  text-align: center;\n}\n\n.kds-hanging-indent {\n  margin: 0px;\n  padding-left: 3em;\n  text-indent: -3em;\n}\n\n.kds-image-caption {\n  font-family: Verdana, Arial, sans-serif;\n  text-align: center;\n}\n\n.kds-image-left {\n  float: left;\n  margin-right: 5px;\n}\n\n.kds-image-right {\n  float: right;\n  margin-left: 5px;\n}\n\n.kds-indent {\n  text-indent: 3em;\n}\n\n.indent {\n  text-indent: 3em;\n}\n\n.kds-nowrap {\n  display: inline-block;\n  white-space: nowrap;\n}\n\n.kds-overline {\n  text-decoration: overline;\n}\n\n.kds-parentheses {\n  font-family: \"Arial Narrow\", Arial, sans-serif;\n  font-size: 300%;\n  font-weight: normal;\n  position: relative;\n  top: 8px;\n}\n\nb.frac {\n  font-weight: normal;\n  font-size: 300%;\n  font-family: arial narrow;\n  position: relative;\n  top: +8px;\n}\n\n.kds-pi {\n  font-family: \"Times New Roman\", serif;\n  font-size: 1.2em;\n}\n\n.newradical {\n  border-collapse: collapse;\n  display: inline-block;\n  *display: inline;\n  vertical-align: middle;\n}\n\n.newradical td {\n  border: 0px;\n  padding: 0px;\n  vertical-align: top;\n}\n\n.newradical td.vinculum {\n  border-top: 1px solid black;\n  padding: 0px;\n  vertical-align: top;\n}\n\n.newradical div.exp {\n  font-size: 0.7em;\n  position: relative;\n  left: 7px;\n  bottom: 5px;\n}\n\n.kds-underline {\n  text-decoration: underline;\n}\n\n.under {\n  text-decoration: underline;\n}\n\n.kds-verdana2t {\n  border: 1px solid white;\n  text-align: left;\n  text-decoration: none;\n  vertical-align: top;\n}\n\n.Verdana2t {\n  vertical-align: top;\n  text-align: left;\n  text-decoration: none;\n  border: 1px solid white;\n}\n\n.kds-whole-number {\n  display: inline-block;\n  font-size: 120%;\n  margin: 0em 0em 0em 0.2em;\n}\n\n.whole {\n  display: inline-block;\n  *display: inline;\n  margin: 0em 0em 0em 0.2em;\n  font-size: 120%;\n}\n\ntable.KdsTable01 {\n  border: 1px solid black;\n  border-collapse: collapse;\n  border-spacing: 0px;\n}\n\ntable.KdsTable01 > tbody > tr > th {\n  background-color: #d3d3d3;\n  border: 1px solid black;\n  font-weight: normal;\n  padding: 2px;\n  text-align: center;\n}\n\ntable.KdsTable01 > tbody > tr > th.bold {\n  background-color: #d3d3d3;\n  border: 1px solid black;\n  font-weight: bold;\n  padding: 2px;\n  text-align: center;\n}\n\ntable.KdsTable01 > tbody > tr > td {\n  border: 1px solid black;\n  padding: 2px;\n  text-align: center;\n}\n\ntable.KdsTable01 > tbody > tr > td.bold {\n  border: 1px solid black;\n  font-weight: bold;\n  padding: 2px;\n  text-align: center;\n}\n\ntable.KdsTable02 {\n  border: 1px solid black;\n  border-collapse: collapse;\n  border-spacing: 0px;\n}\n\ntable.KdsTable02 > tbody > tr > th {\n  background-color: #d3d3d3;\n  border: 1px solid black;\n  font-weight: normal;\n  padding: 2px;\n}\n\ntable.KdsTable02 > tbody > tr > td {\n  border: 1px solid black;\n  padding: 2px;\n}\n\n.table {\n  width: 100%;\n  margin-bottom: 1em;\n  background-color: transparent;\n  border-collapse: collapse;\n}\n\n.table th, .table td {\n  padding: .75em;\n  vertical-align: top;\n  border-top: 1px solid #dee2e6;\n}\n\n.table tbody + tbody {\n  border-top: 2px solid #dee2e6;\n}\n\n.table thead th {\n  vertical-align: bottom;\n  border-bottom: 2px solid #dee2e6;\n}\n\nth {\n  text-align: inherit;\n}\n\n.table-bordered, .table-bordered th, .table-bordered td {\n  border: 1px solid #dee2e6;\n}\n\n.table-bordered thead td, .table-bordered thead th {\n  border-bottom-width: 2px;\n}\n\n.table-striped tbody tr:nth-of-type(odd) {\n  background-color: rgba(0, 0, 0, .05);\n}\n\n.text-center {\n  text-align: center !important;\n}\n\n.h5, h5 {\n  font-size: 1.25em;\n}\n\nh1, h2, h3, h4, h5, h6 {\n  margin-top: 0;\n}\n\n.h1, .h2, .h3, .h4, .h5, .h6, h1, h2, h3, h4, h5, h6 {\n  margin-bottom: .5em;\n  font-family: inherit;\n  font-weight: 500;\n  line-height: 1.2;\n  color: inherit;\n}\n\n/* LEARNOSITY OVERRIDES */\n.lrn_feature h3 {\n  margin-top: 0 !important;\n}\n\n/* PIE OVERRIDES */\n\n#stimulus {\n  width: 50%;\n  float: left;\n}\n\n#item {\n  width: 50%;\n  float: left;\n}\n\n.lrn_width_auto.table {\n  width: auto;\n}\n\n.TEX-S3 {\n  font-family: MJXZERO, MJXTEX-S3 !important;\n}\n\n.TEX-S2 {\n  font-family: MJXZERO, MJXTEX-S2 !important;\n}\n\n.TEX-S1 {\n  font-family: MJXZERO, MJXTEX-S1 !important;\n}\n\n.TEX-I {\n  font-family: MJXZERO, MJXTEX-I !important;\n}\n\nmjx-c.mjx-c2032 {\n  font-family: \'CerebriSans\', serif !important;\n}\n\nmjx-c.mjx-c22, mjx-c.mjx-c27 {\n  font-family: MJXZERO, serif !important\n}\n\ntable {\n  border-collapse: collapse;\n}\n\n.evaluate-bottom-border > :first-child::after {\n  content: \'\';\n  display: block;\n  border-bottom: 1px solid #d3d3d3;\n}"; }
};

export { Player as pie_player };
